/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;


import net.rim.device.api.system.PersistentObject;
import net.rim.device.api.system.PersistentStore;

import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import com.funambol.util.StaticDataHelper;

/**
 * Configuration class for configuration details
 */
public class Configuration
{
    // ------------------------------------------------------------ Constants
    public static final String CONF_KEY_VERSION                 = "VERSION";
    public static final String CONF_KEY_LOG_LEVEL               = "LOG_LEVEL";
    public static final String CONF_KEY_SYNC_URL                = "SYNC_URL";
    public static final String CONF_KEY_USERNAME                = "USERNAME";
    public static final String CONF_KEY_PASSWORD                = "PASSWORD";
    public static final String CONF_KEY_SYNC_CONTACT_FLAG       = "SYNC_CONTACT";
    public static final String CONF_KEY_SYNC_CONTACT_URI        = "CONTACT_URI";
    public static final String CONF_KEY_SYNC_CAL_FLAG           = "SYNC_CAL";
    public static final String CONF_KEY_SYNC_CAL_URI            = "CAL_URI";
    public static final String CONF_KEY_SYNC_NOTE_FLAG          = "SYNC_NOTE";
    public static final String CONF_KEY_SYNC_NOTE_URI           = "NOTE_URI";
    public static final String CONF_KEY_SYNC_TASK_FLAG          = "SYNC_TASK";
    public static final String CONF_KEY_SYNC_TASK_URI           = "TASK_URI";
    public static final String CONF_KEY_POLL_PIM_FLAG           = "POLL_PIM";
    public static final String CONF_KEY_POLL_PIM_TIME           = "POLL_PIM_TIME";
    public static final String CONF_KEY_SYNC_RANGE_PAST         = "RANGE_PAST";
    public static final String CONF_KEY_SYNC_RANGE_FUTURE       = "RANGE_FUTURE";
    public static final String CONF_KEY_SYNC_TYPE               = "SYNC_TYPE";
    public static final String CONF_KEY_BLOCK_INCOMING_INVITES  = "BLOCK_INVITES";
    
    public static final String MAXMSGSIZE                       = "16000";
    public static final String CONFIG_VERSION                   = "1";
    public static String       OLD_CONFIG_VERSION               = "0";
    
    public static String       PROTOCOL;
    public static String       RETRY;
    public static int          logLevel                         = 0; // disabled on plugin, error on api;
    
    public static final int    CONF_OK                          = 0;
    public static final int    CONF_NOTSET                      = -1;
    public static final int    CONF_INVALID                     = -2;

    public static String       syncUrl;
    
    public static String       username;
    public static String       password;
    
    public static boolean      syncCalendar;
    public static boolean      syncContact;
    public static boolean      syncNote;
    public static boolean      syncTask;

    public static String       calendarSourceUri;
    public static String       contactSourceUri;
    public static String       noteSourceUri;
    public static String       taskSourceUri;

    public static boolean      enablePollingPIM;
    public static int          pollingPIMInterval;
    
    public static int          rangePast;
    public static int          rangeFuture;
    
    public static int          syncType;
    
    public static boolean      blockIncomingInvites         = false;
    
    private static void copyDefaults()
    {
        syncUrl              = StaticDataHelper.getServerUriDefault();
        username             = StaticDataHelper.getUserDefault();
        password             = StaticDataHelper.getPasswordDefault();
        
        syncContact          = StaticDataHelper.getSyncContactDefault();
        contactSourceUri     = StaticDataHelper.getContactSourceUriDefault();
        
        syncCalendar         = StaticDataHelper.getSyncCalendarDefault();
        calendarSourceUri    = StaticDataHelper.getCalendarSourceUriDefault();
        
        syncTask             = StaticDataHelper.getSyncTaskDefault();
        taskSourceUri        = StaticDataHelper.getTaskSourceUriDefault();
        
        syncNote             = StaticDataHelper.getSyncNoteDefault();
        noteSourceUri        = StaticDataHelper.getNoteSourceUriDefault();
        
        enablePollingPIM     = StaticDataHelper.getEnablePollingPIMDefault();
        pollingPIMInterval   = StaticDataHelper.getPollingPIMIntervalDefault();
        
        rangePast            = StaticDataHelper.getRangePastDefault();
        rangeFuture          = StaticDataHelper.getRangeFutureDefault();

        syncType             = StaticDataHelper.getSyncTypeDefault();
        blockIncomingInvites = StaticDataHelper.getBlockIncomingInvitesDefault();
    }

    /**
     * This must be called once at the beginning of the application.
     **/
    public static void init(Class c) throws Exception
    {
        StaticDataHelper.init(c,StaticDataHelper.XML_LANGUAGE_FILE);
        copyDefaults();
    }
    
    // ------------------------------------------------------------ Public
    /**
     * Load the current config from the persistent store.
     * @return:
     * <li><b>CONF_OK</b>: if all the data were present in the store.
     * <li><b>CONF_NOTSET</b>: if the store is not present. Configuration
     * remains untouched.
     * <li><b>CONF_INVALID</b>: if the store does not contain valid data.
     * Configuration is reverted to default.
     * Note: if a parameter is not present in the store, the current value is
     * kept for it.
     */
    public static int load()
    {
        Log.info("Loading Configuration data from persistent storage");
        final PersistentObject pobj = PersistentStore.getPersistentObject(SyncClient.CONFIG_KEY);
        try
        {
            final ConfigData contents = (ConfigData) pobj.getContents();
            if (contents == null)
            {
                Log.info("Configuration not set, using current values.");
                return CONF_NOTSET;
            }
            if (contents.get(Configuration.CONF_KEY_VERSION) == null)
            {
                Log.error("Configuration invalid: reverting to default.");
                PersistentStore.destroyPersistentObject(SyncClient.CONFIG_KEY);
                copyDefaults();
                return CONF_INVALID;
            }

            syncUrl = contents.getValue(Configuration.CONF_KEY_SYNC_URL, syncUrl);
            if (Customization.getInstance().stripPortFromSyncAddress()) {
                syncUrl = StringUtil.removePortFromUrl(syncUrl, "http");
            }
            
            username = contents.getValue(Configuration.CONF_KEY_USERNAME, username);
            password = contents.getValue(Configuration.CONF_KEY_PASSWORD, password);
            
            syncContact = contents.getValue(Configuration.CONF_KEY_SYNC_CONTACT_FLAG, syncContact);
            contactSourceUri = contents.getValue(Configuration.CONF_KEY_SYNC_CONTACT_URI, contactSourceUri);
            
            syncCalendar = contents.getValue(Configuration.CONF_KEY_SYNC_CAL_FLAG, syncCalendar);
            calendarSourceUri = contents.getValue(Configuration.CONF_KEY_SYNC_CAL_URI, calendarSourceUri);
            
            syncTask = contents.getValue(Configuration.CONF_KEY_SYNC_TASK_FLAG, syncTask);
            taskSourceUri = contents.getValue(Configuration.CONF_KEY_SYNC_TASK_URI, taskSourceUri);
            
            syncNote = contents.getValue(Configuration.CONF_KEY_SYNC_NOTE_FLAG, syncNote);
            noteSourceUri = contents.getValue(Configuration.CONF_KEY_SYNC_NOTE_URI, noteSourceUri);
            
            enablePollingPIM = contents.getValue(Configuration.CONF_KEY_POLL_PIM_FLAG, enablePollingPIM);
            pollingPIMInterval = contents.getValue(Configuration.CONF_KEY_POLL_PIM_TIME, pollingPIMInterval);
            
            rangePast = contents.getValue(Configuration.CONF_KEY_SYNC_RANGE_PAST, rangePast);
            rangeFuture = contents.getValue(Configuration.CONF_KEY_SYNC_RANGE_FUTURE, rangeFuture);
           
            syncType = contents.getValue(Configuration.CONF_KEY_SYNC_TYPE, syncType);
            
            logLevel = contents.getValue(Configuration.CONF_KEY_LOG_LEVEL, logLevel);
            OLD_CONFIG_VERSION = contents.getValue(Configuration.CONF_KEY_VERSION, CONFIG_VERSION);
            
            blockIncomingInvites = contents.getValue(Configuration.CONF_KEY_BLOCK_INCOMING_INVITES, blockIncomingInvites);
            
            return CONF_OK;
        }
        catch (final ClassCastException cce)
        {
            Log.error("Configuration invalid: reverting to default.");
            PersistentStore.destroyPersistentObject(SyncClient.CONFIG_KEY);
            copyDefaults();
            return -2;
        }
    }

        
    /**
     * Save the current config to the persistent store.
     */
    public static int save(SyncClient app)
    {
        Log.info("Saving Configuration data to persistent storage");
        final PersistentObject pobj = PersistentStore.getPersistentObject(SyncClient.CONFIG_KEY);
        ConfigData contents = (ConfigData) pobj.getContents();
        if (contents == null)
        {
            contents = new ConfigData();
        }

        // This field can be used to check which version of the sw
        // saved the data. If not present, it is assumed it's an
        // old version and the saved config is discarded.
        contents.setValue(Configuration.CONF_KEY_VERSION, CONFIG_VERSION);
        contents.setValue(Configuration.CONF_KEY_LOG_LEVEL, logLevel);

        contents.setValue(Configuration.CONF_KEY_SYNC_URL, syncUrl);
        contents.setValue(Configuration.CONF_KEY_USERNAME, username);
        contents.setValue(Configuration.CONF_KEY_PASSWORD, password);
        
        contents.setValue(Configuration.CONF_KEY_SYNC_CONTACT_FLAG, syncContact);
        contents.setValue(Configuration.CONF_KEY_SYNC_CONTACT_URI, contactSourceUri);
        
        contents.setValue(Configuration.CONF_KEY_SYNC_CAL_FLAG, syncCalendar);
        contents.setValue(Configuration.CONF_KEY_SYNC_CAL_URI, calendarSourceUri);
        
        contents.setValue(Configuration.CONF_KEY_SYNC_TASK_FLAG, syncTask);
        contents.setValue(Configuration.CONF_KEY_SYNC_TASK_URI, taskSourceUri);
        
        contents.setValue(Configuration.CONF_KEY_SYNC_NOTE_FLAG, syncNote);
        contents.setValue(Configuration.CONF_KEY_SYNC_NOTE_URI, noteSourceUri);
        
        contents.setValue(Configuration.CONF_KEY_POLL_PIM_FLAG, enablePollingPIM);
        contents.setValue(Configuration.CONF_KEY_POLL_PIM_TIME, pollingPIMInterval);
        
        contents.setValue(Configuration.CONF_KEY_SYNC_RANGE_PAST, rangePast);
        contents.setValue(Configuration.CONF_KEY_SYNC_RANGE_FUTURE, rangeFuture);
        
        contents.setValue(Configuration.CONF_KEY_SYNC_TYPE, syncType);
        
        contents.setValue(Configuration.CONF_KEY_BLOCK_INCOMING_INVITES, blockIncomingInvites);
        
        synchronized (pobj)
        {
            pobj.setContents(contents);
            pobj.commit();
            app.initAllSourceConfigs(false);
            app.setEnableBlockInvites(blockIncomingInvites);
        }
        return CONF_OK;
    }

    
    public static int append(String key,byte[] value)
    {
        final PersistentObject pobj = PersistentStore.getPersistentObject(SyncClient.CONFIG_KEY);
        ConfigData contents = (ConfigData) pobj.getContents();
        if (contents == null)
        {
            contents = new ConfigData();
        }
        contents.setValue(key,value);
        synchronized (pobj)
        {
            pobj.setContents(contents);
            pobj.commit();
        }
        return CONF_OK;
    }
    
    public static int append(String key,String value)
    {
        final PersistentObject pobj = PersistentStore.getPersistentObject(SyncClient.CONFIG_KEY);
        ConfigData contents = (ConfigData) pobj.getContents();
        if (contents == null)
        {
            contents = new ConfigData();
        }
        contents.setValue(key,value);
        synchronized (pobj)
        {
            pobj.setContents(contents);
            pobj.commit();
        }
        return CONF_OK;
    }
    

    public static byte[] getValue(String key,byte[] value)
    {
        final PersistentObject pobj = PersistentStore.getPersistentObject(SyncClient.CONFIG_KEY);
        try
        {
            final ConfigData contents = (ConfigData) pobj.getContents();
            if (contents == null)
            {
                return null;
            }
            return contents.getValue(key,value);
        }
        catch(final ClassCastException e)
        {}
        return null;
    }
}
