/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import java.util.Hashtable;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;

import java.io.InputStream;
import javax.microedition.pim.PIM;
import javax.microedition.io.HttpConnection;
import javax.microedition.pim.PIMException;

import net.rim.blackberry.api.pdap.BlackBerryPIM;
import net.rim.blackberry.api.pdap.BlackBerryPIMList;
import net.rim.device.api.system.Application;
import net.rim.device.api.system.Bitmap;
import net.rim.device.api.system.RadioInfo;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.FontFamily;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.Screen;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.system.DeviceInfo;
import net.rim.blackberry.api.browser.Browser;

import com.funambol.blackberry.helper.HelperFactory;
import com.funambol.blackberry.helper.PIMItemHelper;
import com.funambol.blackberry.helper.EventHelper;
import com.funambol.blackberry.source.BBPIMSyncSource;
import com.funambol.blackberry.source.BBPIMSyncSourceWithUpdateFix;
import com.funambol.blackberry.source.Poller;
import com.funambol.blackberry.ui.*;
import com.funambol.storage.BlackberryRecordStore;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.spds.CompressedSyncException;
import com.funambol.syncml.spds.DeviceConfig;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncConfig;
import com.funambol.syncml.spds.SyncManager;
import com.funambol.syncml.spds.SyncException;
import com.funambol.util.ConnectionDeniedException;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import com.funambol.util.ConsoleAppender;
import com.funambol.util.FileAppender;
import com.funambol.util.PluginObjectWrapperHanlder;
import com.funambol.util.SimpleXMLParser;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.ConnectionConfig;
import com.funambol.util.ConnectionManager;
import com.funambol.util.BlackberryUtils;
import com.funambol.util.StringUtil;
import com.funambol.syncml.spds.SyncSource;

/**
 * Provides the user interface with the configuration settings, mail
 * synchronization capabilities (polling included), and contacts and calendar
 * events synchronization
 * 
 */
public class SyncClient extends UiApplication implements Runnable
{
    // To control custom features.  Disable for Funambol
    public static final boolean CUSTOMS_ENABLED = false;
    
    public static final int             CUSTOMS_COMPANY_DIRECTORY   = 1;
    public static final int             CUSTOMS_SYNC_RANGE          = 2; //Not wrapped yet
    public static final int             CUSTOMS_UPDATE              = 4;
    public static final int             CUSTOMS_ROLLOVER            = 8;
    public static final int             CUSTOMS_TITLE_HAS_VERSION   = 16;
    public static final int             CUSTOMS_8830_DISABLE_POLL   = 32;
    public static final int             CUSTOMS_ENABLE_PHOTO        = 64;
    public static final int             CUSTOMS_CHECK_DELETES       = 128;


    public static final int CUSTOMS = 
                                        CUSTOMS_COMPANY_DIRECTORY +
                                        CUSTOMS_UPDATE            +
                                        CUSTOMS_SYNC_RANGE        + 
                                        CUSTOMS_ROLLOVER          + 
                                        CUSTOMS_TITLE_HAS_VERSION +
                                        CUSTOMS_8830_DISABLE_POLL +
                                        CUSTOMS_CHECK_DELETES+
                                        0;

    public static final long            CONFIG_KEY                  = 0xbc565a75eda532eL;
    public static final long            CONTACT_STORAGE_KEY         = 0xbc565a75eda5330L;
    public static final long            CALENDAR_STORAGE_KEY        = 0xbc565a75eda5340L;
    public static final long            NOTE_STOREAGE_KEY           = 0xbc565a75eda5350L;
    public static final long            TASK_STORAGE_KEY            = 0xbc565a75eda5360L;
    public static final long            LUID_UID_STORAGE_KEY        = 0xbc565a75eda5370L;
    public static final String          SYNC_CONTACT_CONFIG         = "SYNC_CONTACT_CONFIG";
    public static final String          SYNC_CAL_CONFIG             = "SYNC_CAL_CONFIG";
    public static final String          SYNC_NOTE_CONFIG            = "SYNC_NOTE_CONFIG";
    public static final String          SYNC_TASK_CONFIG            = "SYNC_TASK_CONFIG";
 
    public static final String CONTACT          = "contact";

    /**
     * This property specifies the type of the contact sync source.
     * */
    public static final String CONTACT_TYPE     = "text/x-vcard";

    public static final String          CALENDAR                    = "calendar";
    public static final String          CALENDAR_TYPE               = "text/x-s4j-sife";
    public static final String          NOTE                        = "note";
    public static final String          NOTE_TYPE                   = "text/x-s4j-sifn";
    public static final String          TASK                        = "task";
    public static final String          TASK_TYPE                   = "text/x-s4j-sift";
    
    public static String                VER                         = ""; //Set on startup

    private static final String         ERROR_NOT_AUTHORIZED        = "401";
    private static final String         ERROR_NOT_FOUND             = "404";
    private static final String         ERROR_REFRESH_REQUIRED      = "508";
    private static final String         ERROR_SERVER                = "511";
    
    public static final String          MANUAL_SYNC                 = "manual";
    public static final String          SCHEDULED_SYNC              = "scheduled";
    public static String                syncType;
    private static int                  scheduledAttempt            = 0;
    private static final int            RETRY_POLL_TIME             = 5; //minutes
    private Poller                      retryPoller;
    
    // Used to select which source to sync in synchronize()
    public static final int             SYNC_CONTACTS               = 1;
    public static final int             SYNC_CALENDAR               = 2;
    public static final int             SYNC_TASKS                  = 4;
    public static final int             SYNC_NOTES                  = 8;
    public static final int             SYNC_MAIL                   = 16;
    public static final int             SYNC_ALL                    = 255;

    public static       int             SYNC_SUPPORTED              = SYNC_CONTACTS + 
                                                                      SYNC_CALENDAR +
                                                                      SYNC_TASKS +
                                                                      SYNC_NOTES;
    
    public static final int             REFRESH_FROM_SERVER         = 1;
    public static final int             REFRESH_TO_SERVER           = 2;
    
    private boolean                     inSynchronize               = false;
    
    private DeviceConfig                devconf                     = null;
    private boolean                     doRefresh                   = false;                        
    private boolean                     doCancel                    = false;
    
    private int                         refreshDirection            = 0;
    
    private boolean                     updateFlag                  = false;
    private boolean                     updateRequired              = false;
    private String                      updateOTA                   = null;
    private String                      updateURL                   = null;

    private AppScreen                   appScreen                   = null;
    private Poller                      pollerPIM                   = null;
    private Thread                      thread                      = null;
    private Hashtable                   sources                     = null;
    private SourceConfig                contactSourceConfig         = null;
    private SourceConfig                calendarSourceConfig        = null;
    private SourceConfig                noteSourceConfig            = null;
    private SourceConfig                taskSourceConfig            = null;
    private SyncConfig                  syncConfig                  = null;
    
    private boolean                     logConnectivityError        = false;
    private boolean                     compressionRetry            = false;

    
    static
    {
        StaticDataHelper.setClientVersion("Unknown");
    }
    
    
    /**
     * The default constructor. It opens internally the BlackBerry PIM lists
     * (contacts and calendar events), adds a listener to them and to the mail
     * folders, and pushes the home screen to allow user configuration
     */
    public SyncClient() throws Exception
    {
        // DEBUG only (needed to activate log during conf load
        //Log.initLog(new ConsoleAppender());
        String logFilePath = "file:///store/home/user";
        Log.initLog(new FileAppender(logFilePath, "synclog.txt"));
        // We keep log to trace during startup. The level will be adjusted later
        // (see below)
        Log.setLogLevel(Log.TRACE);

        /*
         * get configurations contents from the store. The class configuration
         * initializes also StaticDataHelper defaults.
         */
        Configuration.init(this.getClass());
        Configuration.load();
        
        syncConfig = new SyncConfig();
        initAllSourceConfigs(true);
        
        /*
         * Check if upgraded application from an old version that does not support
         * the sync of contact with photo to force a slow-sync to sync the photos
         * in just synced contatcs
         * This code will be removed from no upgradable version 
         */
        if(Configuration.OLD_CONFIG_VERSION.equals("20060721")){
            Configuration.append(Configuration.CONF_KEY_VERSION, Configuration.CONFIG_VERSION);
            // In this version we switch from SIF-C to vCard and we assume the
            // name of the vCard sync source is "card"
            Configuration.contactSourceUri = "card";
            contactSourceConfig = SyncClient.initSourceConfig(SYNC_CONTACT_CONFIG,
                                                              CONTACT, CONTACT_TYPE,
                                                              Configuration.contactSourceUri,
                                                              true);
            // This is the first version with photos synchronization
            // If this feature is enabled then we force a slow sync
            if (Customization.getInstance().syncContactPhoto()) {
                contactSourceConfig.setLastAnchor(0);
                contactSourceConfig.setNextAnchor(0);
            }
            SyncClient.saveSourceConfig(SyncClient.SYNC_CONTACT_CONFIG, contactSourceConfig);
        }

        /* Because of a bug in OS 4.6 we shall apply the font to each
         * screen and not globally */
        
        // Set the log level
        Log.setLogLevel(Configuration.logLevel);

        // Create syncsource configs
        sources = new Hashtable();
        BlackBerryPIMList list;
        BBPIMSyncSource syncSource;

        /**
         * Contacts
         */
        try {
            Log.info("Creating Contact sync source");
            list = (BlackBerryPIMList) PIM.getInstance().openPIMList(PIM.CONTACT_LIST,PIM.READ_WRITE);

            boolean updateFix = false;
            String version[] = StaticDataHelper.getOSVersion();
            if (version.length > 1 && version[0].equals("4") && version[1].equals("5")) {
                Log.debug("Version is 4.5, use source with update fix");
                updateFix = true;
            } else {
                Log.debug("Version is NOT 4.5, use standard source");
            }

            if (updateFix) {
                syncSource = new BBPIMSyncSourceWithUpdateFix(this,
                            contactSourceConfig
                            ,list
                            ,HelperFactory.getSIFContactHelper(list)
                            ,SyncClient.CONTACT_STORAGE_KEY
                            ,LUID_UID_STORAGE_KEY
                            );
            } else {
                syncSource = new BBPIMSyncSource(this,
                            contactSourceConfig
                            ,list
                            ,HelperFactory.getSIFContactHelper(list)
                            ,SyncClient.CONTACT_STORAGE_KEY
                            );
            }
            sources.put(new Integer(SyncClient.SYNC_CONTACTS), syncSource);
        }
        catch (Exception e)
        {
            Log.info("Coud not create contacts sync source, disabling contacts");
            SYNC_SUPPORTED = SYNC_SUPPORTED & ~SYNC_CONTACTS;
            Configuration.syncContact = false;
        }

        /**
         * Calendar
         */
        try {
            Log.info("Creating Event sync source");
            list = (BlackBerryPIMList) PIM.getInstance().openPIMList(PIM.EVENT_LIST,PIM.READ_WRITE);
            PIMItemHelper eventhelper = HelperFactory.getSIFEventHelper(list, Configuration.rangePast,
                                                                        Configuration.rangeFuture);
            syncSource = new BBPIMSyncSource(this,
                            calendarSourceConfig
                            ,list
                            ,eventhelper
                            ,SyncClient.CALENDAR_STORAGE_KEY
                            );
            sources.put(new Integer(SyncClient.SYNC_CALENDAR), syncSource);
        }
        catch (Exception e)
        {
            Log.info("Coud not create calendar sync source, disabling calendar");
            SYNC_SUPPORTED = SYNC_SUPPORTED & ~SYNC_CALENDAR;
            Configuration.syncCalendar = false;
        }

        /**
         * Tasks
         */
        try {
            Log.info("Creating Task sync source");
            list = (BlackBerryPIMList) PIM.getInstance().openPIMList(PIM.TODO_LIST,PIM.READ_WRITE);
            PIMItemHelper helper = HelperFactory.getSIFTaskHelper(list);
            helper.setDeleteForUpdate(true);
            syncSource = new BBPIMSyncSource(this,
                            taskSourceConfig
                            ,list
                            ,helper
                            ,SyncClient.TASK_STORAGE_KEY
                            );
            sources.put(new Integer(SyncClient.SYNC_TASKS), syncSource);
        }
        catch (Exception e)
        {
            Log.info("Coud not create tasks sync source, disabling tasks");
            SYNC_SUPPORTED = SYNC_SUPPORTED & ~SYNC_TASKS;
            Configuration.syncTask = false;
        }

        /**
         * Note
         */
        try {
            Log.info("Creating Note sync source");
            list = (BlackBerryPIMList) PIM.getInstance().openPIMList(BlackBerryPIM.MEMO_LIST,PIM.READ_WRITE);
            syncSource = new BBPIMSyncSource(this,
                            noteSourceConfig
                            ,list
                            ,HelperFactory.getSIFNoteHelper(list)
                            ,SyncClient.NOTE_STOREAGE_KEY
                            );
            sources.put(new Integer(SyncClient.SYNC_NOTES), syncSource);
        }
        catch (Exception e)
        {
            Log.info("Coud not create notes sync source, disabling notes");
            SYNC_SUPPORTED = SYNC_SUPPORTED & ~SYNC_NOTES;
            Configuration.syncNote = false;
        }

        // Turn on polling, if necessary
        if (Configuration.enablePollingPIM)
        {
            enablePollingPIM();
        }

        // Initalize the interfaces
        appScreen = new AppScreen(this);
        long lastSync = 
                Math.max(
                        Math.max(
                                calendarSourceConfig.getNextAnchor(),
                                contactSourceConfig.getNextAnchor()
                                ),
                        Math.max(
                                taskSourceConfig.getNextAnchor(),
                                noteSourceConfig.getNextAnchor()
                                )
                        );
        if (lastSync > 0)
        {
            appScreen.setLastSync(lastSync);
        }

        if (contactSourceConfig.getNextAnchor() == 0)
            appScreen.getSyncScreen().displayStatus(SYNC_CONTACTS,
                                      StaticDataHelper.getLanguage("sync_unsynchronized"));
        if (calendarSourceConfig.getNextAnchor() == 0)
            appScreen.getSyncScreen().displayStatus(SYNC_CALENDAR,
                                      StaticDataHelper.getLanguage("sync_unsynchronized"));
        if (taskSourceConfig.getNextAnchor() == 0)
            appScreen.getSyncScreen().displayStatus(SYNC_TASKS,
                                      StaticDataHelper.getLanguage("sync_unsynchronized"));
        if (noteSourceConfig.getNextAnchor() == 0)
            appScreen.getSyncScreen().displayStatus(SYNC_NOTES,
                                      StaticDataHelper.getLanguage("sync_unsynchronized"));

        /*
         * The UI engine must be accessed holding the event lock object shared
         * with the application's main thread, otherwise an
         * IllegalStateException is thrown (the event lock is held here where
         * the screen is pushed in the foreground)
         */
        synchronized(SyncClient.getEventLock())
        {
            MainScreen nextScreen = appScreen;
            if (Customization.getInstance().loginScreenRequired()) {

                if (StringUtil.isNullOrEmpty(Configuration.username) ||
                    StringUtil.isNullOrEmpty(Configuration.password))
                {
                    nextScreen = new LoginScreen(this);
                }
            }

            pushScreen(nextScreen);

            if (SyncClient.checkCustom(SyncClient.CUSTOMS_UPDATE))
            {
                if (checkUpdate())
                {
                    SyncClient.showMessage(StaticDataHelper.getLanguage("message_update-required"));
                    SyncClient.getApplication().requestForeground();
                }
                else if (updateFlag)
                {
                    SyncClient.showMessage(StaticDataHelper.getLanguage("message_update-available"));
                    SyncClient.getApplication().requestForeground();
                }
            }
        }
    }
    
    /**
     * Helper function to prompt the user for a yes/no answer
     * 
     * @param question
     * @param defaultyes
     * @return                          True on yes
     */
    public boolean askYesNoQuestion(String question, boolean defaultyes)
    {
        synchronized (SyncClient.getEventLock())
        {
            return Dialog.YES == Dialog.ask(Dialog.D_YES_NO,question,defaultyes ? Dialog.YES : Dialog.NO);
        }
    }

    /**
     * Helper function to prompt to continue or cancel
     * 
     * @param message
     * @return  True for next
     */
    public boolean promptNext(String message)
    {
        String[] buttons = new String[]{
            StaticDataHelper.getLanguage("dialog_continue"),
            StaticDataHelper.getLanguage("dialog_abort")
        };

        int [] values = new int [] {
            0, 1
        };

        synchronized (SyncClient.getEventLock())
        {
            return 0 == Dialog.ask(message, buttons, values, 0);
        }
    }
    
    /**
     * Helper function to display a simple notice
     * 
     * @param message
     */
    public static void showMessage(String message)
    {
        synchronized (SyncClient.getEventLock())
        {
            final Dialog dialog = new Dialog(
                Dialog.D_OK,
                message,
                1,
                Bitmap.getPredefinedBitmap(Bitmap.EXCLAMATION),
                Manager.LEAVE_BLANK_SPACE);
            SyncClient.showScreen(dialog);
        }
    }

    /**
     * Push a screen to the forefront
     * 
     * @param scr       The screen to push
     */
    public static void showScreen(Screen scr)
    {

        class ShowScreenRequest implements Runnable
        {
            private Screen screen;
            public ShowScreenRequest(Screen scr)
            {
                screen = scr;
            }
            
            public void run()
            {
                synchronized (SyncClient.getEventLock())
                {
                    if (screen.isDisplayed())
                    {
                        SyncClient.getUiApplication().popScreen(screen);
                    }
                    SyncClient.getUiApplication().pushScreen(screen);
                }                
            }
        }
        
        if (SyncClient.getUiApplication().hasEventThread())
        {
            synchronized (SyncClient.getEventLock())
            {
                if (scr.isDisplayed())
                {
                    SyncClient.getUiApplication().popScreen(scr);
                }
                SyncClient.getUiApplication().pushScreen(scr);
            }
        }
        else
        {
            SyncClient.getUiApplication().invokeLater(new ShowScreenRequest(scr));
        }
    }
    
    /**
     * Pull a screen to the back
     * 
     * @param scr       The screen to hide
     */
    public static void hideScreen(MainScreen scr)
    {
        class HideScreenRequest implements Runnable
        {
            private MainScreen screen;
            public HideScreenRequest(MainScreen scr)
            {
                screen = scr;
            }
            public void run()
            {
                synchronized (SyncClient.getEventLock())
                {
                    if (screen.isDisplayed())
                    {
                        SyncClient.getUiApplication().popScreen(screen);    
                    }
                }                
            }
        }
        
        SyncClient.getUiApplication().invokeLater(new HideScreenRequest(scr));
    }

    /**
     * Initalize all syncsource configs.  
     * 
     * @param read      Pass true to read from the device persistent storage.  Forwarded to initSourceConfig
     */
    public void initAllSourceConfigs(boolean read)
    {
        long last = 0;
        long next = 0;
        boolean restore;
        

        restore = false;
        if (contactSourceConfig != null)
        {
            last = contactSourceConfig.getLastAnchor();
            next = contactSourceConfig.getNextAnchor();
            restore = true;
        }
        contactSourceConfig  = SyncClient.initSourceConfig(SyncClient.SYNC_CONTACT_CONFIG,
                                                           CONTACT, CONTACT_TYPE,
                                                           Configuration.contactSourceUri,
                                                           read);
        if (restore)
        {
            contactSourceConfig.setLastAnchor(last);
            contactSourceConfig.setNextAnchor(next);
        }
        SyncClient.saveSourceConfig(SyncClient.SYNC_CONTACT_CONFIG, contactSourceConfig);
        
        
        restore = false;
        if (calendarSourceConfig != null)
        {
            last = calendarSourceConfig.getLastAnchor();
            next = calendarSourceConfig.getNextAnchor();
            restore = true;
        }
        calendarSourceConfig = SyncClient.initSourceConfig(SyncClient.SYNC_CAL_CONFIG,
                                                           CALENDAR, CALENDAR_TYPE,
                                                           Configuration.calendarSourceUri,
                                                           read);
        if (restore)
        {
            calendarSourceConfig.setLastAnchor(last);
            calendarSourceConfig.setNextAnchor(next);
        }
        SyncClient.saveSourceConfig(SyncClient.SYNC_CAL_CONFIG, calendarSourceConfig);
        
        
        restore = false;
        if (noteSourceConfig != null)
        {
            last = noteSourceConfig.getLastAnchor();
            next = noteSourceConfig.getNextAnchor();
            restore = true;
        }
        noteSourceConfig     = SyncClient.initSourceConfig(SyncClient.SYNC_NOTE_CONFIG,
                                                           NOTE, NOTE_TYPE,
                                                           Configuration.noteSourceUri,
                                                           read);
        if (restore)
        {
            noteSourceConfig.setLastAnchor(last);
            noteSourceConfig.setNextAnchor(next);
        }
        SyncClient.saveSourceConfig(SyncClient.SYNC_NOTE_CONFIG, noteSourceConfig);
    
        
        restore = false;
        if (taskSourceConfig != null)
        {
            last = taskSourceConfig.getLastAnchor();
            next = taskSourceConfig.getNextAnchor();
            restore = true;
        }
        taskSourceConfig     = SyncClient.initSourceConfig(SyncClient.SYNC_TASK_CONFIG,
                                                           TASK, TASK_TYPE,
                                                           Configuration.taskSourceUri,
                                                           read);
        if (restore)
        {
            taskSourceConfig.setLastAnchor(last);
            taskSourceConfig.setNextAnchor(next);
        }
        SyncClient.saveSourceConfig(SyncClient.SYNC_TASK_CONFIG, taskSourceConfig);
    }
    
    /**
     * Initalize a specific syncsource config
     * 
     * @param storageKey
     * @param name
     * @param type
     * @param remoteUri
     * @param read              True will read from the Configuration.  False will create a new config
     * @return
     */
    private static SourceConfig initSourceConfig(final String storageKey, final String name,
                                                 final String type, final String remoteUri,
                                                 boolean read)
    {
        Log.info("Initialize SourceConfig ["+name+"]");
        SourceConfig ret = new SourceConfig(name, type, remoteUri);

        if (CONTACT.equals(name)) {
            // this is hacky, but the way the code is organized today we need
            // to change sync sources encoding for all non-SIF sources
            Log.info("Setting encoding to NONE for source " + name);
            ret.setEncoding(com.funambol.syncml.spds.SyncSource.ENCODING_NONE);
        }

        if (read)
        {
            try
            {
                byte[] byte_array = Configuration.getValue(storageKey,new byte[0]);
                if (byte_array!=null)
                {
                    DataInputStream temp = new DataInputStream(new ByteArrayInputStream(byte_array));
                    ret.deserialize(temp);
                    temp.close();
                }
            }
            catch(final Exception e)
            {
                Log.error("Exception while initialization (read) of SourceConfig ["+name+"]");
            }
        }
        return ret;
    }

    /**
     * Saves a syncsource config to storage
     * 
     * @param storageKey
     * @param config
     */
    private static void saveSourceConfig(final String storageKey, SourceConfig config)
    {
        Log.trace("Storing SourceConfig ["+config.getName()+"]");
        try
        {
            ByteArrayOutputStream buff = new ByteArrayOutputStream(512);
            DataOutputStream temp = new DataOutputStream(buff);
            config.serialize(temp);
            Configuration.append(storageKey,buff.toByteArray());
            temp.close();
        }
        catch(final Exception e)
        {
            Log.error("Exception while storing SourceConfig ["+config.getName()+"]");
        }
    }
   
    /**
     * Immediately disable autosync
     */ 
    public void disablePollingPIM()
    {
        if (pollerPIM != null && pollerPIM.isAlive())
        {
            pollerPIM.disable();
            pollerPIM = null;
        }
    }

    /**
     * Enable autosync.  This will reset the timer
     */
    public void enablePollingPIM()
    {
        disablePollingPIM();
        
        try
        {
            pollerPIM = new Poller(this, Configuration.pollingPIMInterval,Configuration.enablePollingPIM);
            pollerPIM.start();
        }
        catch (final Throwable t)
        {
            Log.error("Exception " + t.getMessage()+ " creating a new SyncClient object for the Poller: "
                      + t.toString());
            return;
        }
    }

    /**
     * Helper function to build the DevConf
     * 
     * @return  A DeviceConfig
     */
    private DeviceConfig getDeviceConfig()
    {
        if (devconf != null)
        {
            return devconf;
        }
        devconf = new DeviceConfig();
        devconf.man = DeviceInfo.getManufacturerName();
        devconf.mod = DeviceInfo.getDeviceName();
        devconf.swv = VER;
        devconf.hwv = "" + DeviceInfo.getPlatformVersion();
        devconf.devID = "fbb-" + DeviceInfo.getDeviceId();
        devconf.maxMsgSize = 32*1024;
        devconf.loSupport = false;
        devconf.utc = true;
        devconf.nocSupport = true;
        return devconf;
    }

    
    /**
     * return error message
     * @param error
     * @param remoteDBType
     * @param sourceUri
     */
    private String getSyncError(String error, String remoteDBType, String sourceUri)
    {
        if (ERROR_NOT_AUTHORIZED.equals(error))
        {
            return StaticDataHelper.getLanguage("message_auth-failed");
        }
        else if (ERROR_NOT_FOUND.equals(error))
        {
            return StaticDataHelper.getLanguage("message_syncsource-not-found") + ": " + sourceUri;
        }
        else if (ERROR_SERVER.equals(error))
        {
            return StaticDataHelper.getLanguage("message_internal-server-error");
        }
        else if (error.indexOf(ERROR_REFRESH_REQUIRED) >= 0)
        {// 508
            for (int x = 1; x < SYNC_ALL; x *= 2)
            {
                BBPIMSyncSource ss = (BBPIMSyncSource) sources.get(new Integer(x));
                if (remoteDBType.equals(ss.getName()))
                {
                        ss.setLastAnchor(01);
                }
            }
            return StaticDataHelper.getLanguage("message_refresh");
        }
        
        else if (error != null)
        {
            Log.error(error);
            return StaticDataHelper.getLanguage("message_generic-error");
        }
        else
        {// error == null
            // if we return just 'error', the string "null" is printed on the
            // device's screen!"
            Log.error(error);
            return StaticDataHelper.getLanguage("message_generic-error");
        }
    }

    
    /**
     * Overridden run method of Runnable interface. A seperate thread is created
     * to synchronize data with sync server.
     * 
     * 'thread' is always the same Thread object in this class, representing the
     * runnable class SyncClient itself. It is created by choosing the
     * Synchronize menu item
     * 
     * @param void
     * @return void
     */
    public void run()
    {
        try
        {
            final Thread current = Thread.currentThread();
            if (this.thread == current)
            {
                synchronized (this)
                {
                    synchronize();
                }
            }
        }
        catch (final Exception ex)
        {
            Log.error("SyncClient.run(): Exception -> " + ex.toString());
            appScreen.getSyncScreen().displayFailure(
                        StaticDataHelper.getLanguage("sync_exception") + " " + ex);
        }
    }

    /**
     * Get the AppScreen
     */
    public AppScreen getAppScreen()
    {
        return appScreen;
    }

    /**
     * Initiate a refresh for some data types
     * 
     * @param syncMask  The mask of the types to sync
     */
    public void refresh(int syncMask, int direction)
    {
        doRefresh = true;
        refreshDirection = direction;
        synchronize(syncMask);  
    }
    
    /**
     * Invoked when the synchronize button in the menu of the home screen is
     * clicked. In this case a thread representing this runnable class
     * (SyncClient) is created and the run() method implemented by SyncClient is
     * fired.
     * 
     * @see run()
     */
    public void synchronize()
    {
        synchronize(SYNC_SUPPORTED);
    }

    
    /**
     * This method starts a synchronization on the specified sources, if they
     * are also configured by the user. It is used both for user and server
     * initialized sync
     * 
     * @param syncMask
     *        A bit mask of sources to sync
     */
    public void synchronize(int syncMask)
    {
        if (ConnectionConfig.isUserConfirmationNeeded()) {
            Log.debug("[SyncClient.synchronize]Setting connection listener for this application");
            ConnectionManager.getInstance().setConnectionListener(appScreen);
            Log.debug("[SyncClient.synchronize]Connection listener set");
        } else {
            Log.debug("[SyncClient.synchronize]Using Default BasicConnectionListener");
        }

        
       
        if (Configuration.username == null || Configuration.username.length() == 0 || 
                        Configuration.password == null || Configuration.password.length() == 0)
        {
            showMessage(StaticDataHelper.getLanguage("message_login-required"));
            return;
        }
        
        doCancel = false;
        Log.info("SyncClient.synchronize(int): Entering - 'inSynchronize' is " + inSynchronize);
        try
        {
            if (!Configuration.syncContact && !Configuration.syncCalendar
                    && !Configuration.syncTask && !Configuration.syncNote)
            {
                showMessage(
                    StaticDataHelper.getLanguage("message_nothing-to-sync")
                );              
                return;
            }
            
            boolean signal = BlackberryUtils.isDataConnectionAvailable();
            if (!signal)
            {
                if (RadioInfo.getState() == RadioInfo.STATE_OFF)
                {
                    showMessage(
                        StaticDataHelper.getLanguage("message_radio-off")
                    );
                }
                else
                {
                    showMessage(
                        StaticDataHelper.getLanguage("message_no-signal")
                    );
                }
                return;
            }
            if (SyncClient.checkCustom(SyncClient.CUSTOMS_CHECK_DELETES))
            {
                if (syncMask != 0)
                {
                    syncMask = checkDeletes(syncMask);
                }
            }
            if (syncMask == 0)
            {
                // TODO print message - nothing enabled
            }
            else
            { 
                if (doCancel)
                        return;
                
                showScreen(appScreen.getSyncScreen());

                final SyncThread syncThread = new SyncThread(syncMask);
                // Set a low priority on the sync thread to avoid
                // device slowness
                syncThread.setPriority(Thread.MIN_PRIORITY);

                syncThread.start();
            }// end if
        }
        catch (final Exception e)
        {
            appScreen.getSyncScreen().displayFailure(e.getMessage());
        }
    }

    
    /**
     * The main procedure for the sync thread
     * 
     * @param syncMask
     * @throws Exception
     */
    public void syncThreadRun(int syncMask) throws Exception
    {

        //Uncomment this line to enable automatic wifi usage at every sync
        //ConnectionConfig.removeSavedConfig();
        Log.info(ConnectionConfig.getConfigsDescription());
        Log.info(ConnectionConfig.getWorkingConfigurationDescription());
        //StaticDataHelper.getErrors().clearErrors();
        
        syncConfig.syncUrl           = Configuration.syncUrl;
        syncConfig.lastServerUrl     = Configuration.syncUrl;
        syncConfig.userName          = Configuration.username;
        syncConfig.password          = Configuration.password;
        syncConfig.userAgent         = "Funambol BlackBerry Plug-in " + BBVersion.VERSION;
        syncConfig.deviceConfig      = getDeviceConfig();
        syncConfig.forceCookies      = false;

        SyncManager manager          = new SyncManager(syncConfig);
        
        int syncMode = SyncML.ALERT_CODE_FAST;
        boolean showTCPAlert = false;
        String status;

        if (doRefresh)
        {
            if (refreshDirection == SyncClient.REFRESH_FROM_SERVER)
            {
                syncMode = SyncML.ALERT_CODE_REFRESH_FROM_SERVER;
            }
            else if (refreshDirection == SyncClient.REFRESH_TO_SERVER)
            {
                syncMode = SyncML.ALERT_CODE_REFRESH_FROM_CLIENT;
            }
            else
            {
                Log.error("Invalid refresh code");
                throw new Exception("Invalid refresh code");
            }
            doRefresh = false;
            status = StaticDataHelper.getLanguage("sync_recover-in-progress");
        }
        else
        {
            switch (Configuration.syncType){
                case StaticDataHelper.SYNC_TYPE_TWO_WAY:
                default:
                        syncMode = SyncML.ALERT_CODE_FAST;
                break;
                
                case StaticDataHelper.SYNC_TYPE_ONE_WAY_TO_CLIENT:
                        syncMode = SyncML.ALERT_CODE_ONE_WAY_FROM_SERVER;
                break;
                
                case StaticDataHelper.SYNC_TYPE_ONE_WAY_TO_SERVER:
                        syncMode = SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT;
                break;
            }
            
            status = StaticDataHelper.getLanguage("sync_in-progress");
        }
        
        if (!compressionRetry)
        {
            appScreen.getSyncScreen().displayStatus(
                    StaticDataHelper.getLanguage("sync_init")
                );
            appScreen.getSyncScreen().displayStatus(
                    SYNC_SUPPORTED, StaticDataHelper.getLanguage("sync_init")
                );
        }

        appScreen.getSyncScreen().setGlobalStatus(" ");

        if (!Configuration.syncContact)
                syncMask &= ~SYNC_CONTACTS;
        if (!Configuration.syncCalendar)
                syncMask &= ~SYNC_CALENDAR;
        if (!Configuration.syncTask)
                syncMask &= ~SYNC_TASKS;
        if (!Configuration.syncNote)
                syncMask &= ~SYNC_NOTES;
        
        syncMask = syncMask & SYNC_SUPPORTED;
        
        for (int x = 1; x <= SYNC_SUPPORTED; x*=2)
        {
             if ((syncMask & x) != 0)
             {
                appScreen.getSyncScreen().displayStatus(x,StaticDataHelper.getLanguage("sync_wait"));
                appScreen.getSyncScreen().setClientCount(x, 0, 0);
                appScreen.getSyncScreen().setServerCount(x, 0, 0);
             }
             else
             {
                appScreen.getSyncScreen().displayDisabled(x,StaticDataHelper.getLanguage("sync_disabled"));
                appScreen.getSyncScreen().setClientCount(x, 0, 0);
                appScreen.getSyncScreen().setServerCount(x, 0, 0);
             }
        }

        if (SyncClient.checkCustom(SyncClient.CUSTOMS_UPDATE))
        {
            if (!compressionRetry)
            {
                appScreen.getSyncScreen().displayStatus(
                        StaticDataHelper.getLanguage("sync_check-update")
                );
                
                if (checkUpdate())
                {
                    synchronized (SyncClient.getEventLock())
                    {
                        SyncClient.showScreen(appScreen);
                        SyncClient.showMessage(StaticDataHelper.getLanguage("message_update-required"));
                        SyncClient.getApplication().requestForeground();
                    }
                    return;
                }
                else if (updateFlag)
                {
                    synchronized (SyncClient.getEventLock())
                    {
                        SyncClient.showMessage(StaticDataHelper.getLanguage("message_update-available"));
                        SyncClient.getApplication().requestForeground();
                    }
                }                
            }
        }

        appScreen.getSyncScreen().displayStatus(status);
        
        int numfail = 0;
        for (int thisSyncMask = 1; thisSyncMask <= SYNC_SUPPORTED; thisSyncMask*=2)
        {
            String type = null;
            String uri = null;
            SourceConfig ssc = null;
            String sscKey = null;
            boolean doSync = false;
            
            if ((syncMask & thisSyncMask) != 0)
            {
                switch (thisSyncMask)
                {
                    case SyncClient.SYNC_CONTACTS:
                            type = "contacts";
                            uri = Configuration.contactSourceUri;
                            ssc = contactSourceConfig;
                            sscKey = SyncClient.SYNC_CONTACT_CONFIG;
                            doSync = Configuration.syncContact;
                            break;
                            
                    case SyncClient.SYNC_CALENDAR:
                            type = "calendar";
                            uri = Configuration.calendarSourceUri;
                            ssc = calendarSourceConfig;
                            sscKey = SyncClient.SYNC_CAL_CONFIG;
                            doSync = Configuration.syncCalendar;
                            break;
                            
                    case SyncClient.SYNC_TASKS:
                            type = "tasks";
                            uri = Configuration.taskSourceUri;
                            ssc = taskSourceConfig;
                            sscKey = SyncClient.SYNC_TASK_CONFIG;
                            doSync = Configuration.syncTask;
                            break;
                            
                    case SyncClient.SYNC_NOTES:
                            type = "notes";
                            uri = Configuration.noteSourceUri;
                            ssc = noteSourceConfig;
                            sscKey = SyncClient.SYNC_NOTE_CONFIG;
                            doSync = Configuration.syncNote;
                            break;
                            
                    default:
                            doSync = false;
                            break;
                }
                if (doSync)
                {
                    if (doCancel)
                    {
                        appScreen.getSyncScreen().displayFailure(thisSyncMask,
                                StaticDataHelper.getLanguage("sync_cancelled")
                        );
                    }
                    else
                    {
                        appScreen.getSyncScreen().displayStatus(
                                    thisSyncMask,
                                    StaticDataHelper.getLanguage("sync_syncing"));
                        
                        BBPIMSyncSource source = (BBPIMSyncSource) sources.get(new Integer(thisSyncMask));
                        source.setConfig(ssc);
                        
                        try
                        {
                            manager.setFlagSendDevInf(); // TODO not always necessary
                            manager.sync(source, syncMode);
                            appScreen.getSyncScreen().setGlobalStatus(" ");

                            if (source.getErrorCount() != 0)
                            {
                                appScreen.getSyncScreen().displayError(thisSyncMask,
                                        StaticDataHelper.getLanguage("sync_complete-errors")
                                        );
                            }
                            else if (source.getWarningCode()!= null)
                            {
                                appScreen.getSyncScreen().displayWarning(thisSyncMask,
                                        StaticDataHelper.getLanguage("sync_complete-warnings")
                                        );
                            }
                            else
                            {
                                appScreen.getSyncScreen().displayStatus(thisSyncMask,
                                        StaticDataHelper.getLanguage("sync_complete")
                                        );
                            }
                            
                            SyncClient.saveSourceConfig(sscKey,ssc);
                        } catch (final SyncException e) {
                            numfail++;
                            source.restoreTrackingData();
                            String message = "";
                            String message2 = "";
                            try {
                                message = getSyncError(e.getMessage(), type, uri);
                                message2 = message;
                            } catch (Exception e2) {
                                message = StaticDataHelper.getLanguage("message_no-error");
                                message2 = "Exception type: " + e.getClass().getName();
                            }
                            
                            boolean compressError = (e instanceof CompressedSyncException);
                            
                            if (compressError && !SyncClient.this.compressionRetry)
                            {
                                message2 += " - Retrying without compression";
                                Log.info(message2);
                            }
                            
                            if (!compressError || (compressError && SyncClient.this.compressionRetry))
                            {
                                appScreen.getSyncScreen().displayFailure(thisSyncMask,
                                        StaticDataHelper.getLanguage("sync_failed") + " - " + message
                                );
                                Log.error(message2);
                            }
                            
                            if (compressError)
                            {
                                throw new CompressedSyncException(e.getMessage());
                            }
                            
                            int code = e.getCode();
                            if (code == SyncException.READ_SERVER_RESPONSE_ERROR || 
                                code == SyncException.WRITE_SERVER_REQUEST_ERROR || 
                                code == SyncException.CONN_NOT_FOUND
                                )
                            {
                                logConnectivityError = true;
                            }
                            
                             
                              if( code == SyncException.CONNECTION_BLOCKED_BY_USER) {
                                   showTCPAlert = true ;
                               }
                             
                            
                            
                        }
                        appScreen.getSyncScreen().setGlobalStatus(" ");
                        source = null;
                    }
                }
            }
        }

        doCancel = false;
        
        // -- END OF A SYNCHRONIZATION
        if (numfail > 0 )
        {
            appScreen.getSyncScreen().displayFailure(StaticDataHelper.getLanguage("sync_complete-errors"));
            showScreen(appScreen.getSyncScreen());
            Application.getApplication().requestForeground();
            
            //TODO: fix this, it's horrible to use a flag like this.
            
            if (showTCPAlert) {
                Log.debug("showing tcp settings alert!");
                SyncClient.showMessage(StaticDataHelper.getLanguage("message_enter-TCP-settings"));
                           
            }
        }
        else
        {
            appScreen.getSyncScreen().displaySuccess(StaticDataHelper.getLanguage("sync_complete"));
        }
        
        if (logConnectivityError)
        {
            if (syncType.equals(MANUAL_SYNC)) {
                Log.error("Manual sync: Connection attempts failed.");
                Log.error(StaticDataHelper.getLanguage("message_connectivity"));
                logConnectivityError = false;
            } else if (syncType.equals(SCHEDULED_SYNC)) {

                if (retryPoller != null) {
                    retryPoller.disable();
                }

                if (scheduledAttempt < 3) {
                    scheduledAttempt++;
                    Log.error("Scheduled sync: Connection attempt failed. " + "Try again between "
                            + RETRY_POLL_TIME + " minutes");

                    retryPoller = new Poller(this, RETRY_POLL_TIME, true);
                    retryPoller.start();
                } else {
                    retryPoller.disable();
                    retryPoller = null;
                    scheduledAttempt = 0;
                    Log.error(StaticDataHelper.getLanguage("message_connectivity"));
                    logConnectivityError = false;
                }
            }
        }
        
        
    }// end run()
   
    
    /**
     * Cancel the current sync
     */
    public void cancelSync()
    {
        doCancel = true;
        if (inSync())
        {
            
            appScreen.getSyncScreen().displayStatus(StaticDataHelper.getLanguage("sync_cancelling"));
            
        }
    }
    
    /**
     * Check if the current sync should be cancelled
     * 
     * @return
     */
    public boolean isCancelled()
    {
        return doCancel;
    }
    
    /**
     * Requests minimization of the program
     **/
    public void minimize()
    {
        getApplication().requestBackground();
    }

    /**
     * Returns true if there is current a sync in progress
     **/
    public boolean inSync()
    {
        return inSynchronize;
    }
    
    /**
     * 
     * @return true if there is a required update
     */
    public boolean checkUpdate()
    {
        final String UPDATE_ROOT_TAG = "sync";
        final String BB_UPDATE_TAG = "blackberry";
        final String VERSION_TAG = "version";
        final String REQUIRED_TAG = "required";
        final String URL_TAG = "url";
        final String OTA_TAG = "ota";
        final String VER_SEP = ".";

        if (RadioInfo.getState() == RadioInfo.STATE_OFF ||
            RadioInfo.getSignalLevel() == RadioInfo.LEVEL_NO_COVERAGE)
        {
            return false;
        }

        // TODO Strip all trailing directories from sync server uri
        // and append update.xml
        String url = StaticDataHelper.getLanguage("update_xml");
   
        HttpConnection c = null;
        InputStream is = null;
        int rc;
       
        try {
            Log.info(url);
            c = (HttpConnection)ConnectionManager.getInstance().open(url);
            // Getting the response code will open the connection,
            // send the request, and read the HTTP response headers.
            // The headers are stored until requested.
   
            rc = c.getResponseCode();
            if (rc != HttpConnection.HTTP_OK) 
            {
                return false;
            }
            is = c.openInputStream();
   
            SimpleXMLParser parser = new SimpleXMLParser();
            parser.addHashtableValueTag(BB_UPDATE_TAG);
               
            Hashtable output = parser.parseData(UPDATE_ROOT_TAG,is);
            Hashtable bb = (Hashtable)output.get(BB_UPDATE_TAG);
            if (bb!=null)
            {
                String version = SyncClient.VER;
                String fversion = (String)bb.get(VERSION_TAG);
   
                //String required = (String)bb.get(REQUIRED_TAG);
                   
                int vpos = 0;
                int vfpos = 0;
                do
                {
                    vpos = version.indexOf(VER_SEP,0);
                    vfpos = fversion.indexOf(VER_SEP,0);
                    if (vpos < 0 && version.length()>0)
                    {
                        vpos = version.length();
                    }
                    if (vfpos < 0 && fversion.length()>0)
                    {
                        vfpos = fversion.length();
                    }
   
                    if (vpos > 0 && vfpos > 0)
                    {
                        int val = Integer.parseInt(version.substring(0,vpos));
                        int fval = Integer.parseInt(fversion.substring(0,vfpos));
                        if (val < fval)
                        {
                            SyncClient.this.updateFlag = true;
                            break;
                        }
                        else if (val > fval)
                        {
                            SyncClient.this.updateFlag = false;
                            break;
                        }
                        if (vpos<version.length())
                        {
                            version = version.substring(vpos+1);
                        }
                        else
                        {
                            vpos = -1;
                        }
                        if(vfpos<fversion.length())
                        {
                            fversion = fversion.substring(vfpos+1);
                        }
                        else
                        {
                            vfpos = -1;
                        }
                    }
                }
                while(vpos > 0 && vfpos > 0);
                   
                if (SyncClient.this.updateFlag)
                {
                    SyncClient.this.updateOTA = (String)bb.get(OTA_TAG);
                    SyncClient.this.updateURL = (String)bb.get(URL_TAG);
                    SyncClient.this.updateRequired = !((String)bb.get(REQUIRED_TAG)).equals("0");
                }
            }
            else
            {
                Log.info("BlackBerry info set is null");
            }
            output = null;
            bb = null;
        } 
        catch (ClassCastException cce) 
        {
            Log.info(cce.toString());
        } 
        catch (Throwable t) 
        {
            Log.info(t.toString());
        } 
        finally 
        {
            if (is != null)
                try { is.close(); } catch (Exception e) {}
            if (c != null)
                try { c.close();  } catch (Exception e) {}
        }
   
        Log.info("updateFlag ["+SyncClient.this.updateFlag+"] updateOTA ["+
                 SyncClient.this.updateOTA+"] updateURL ["+SyncClient.this.updateURL+
                 "] updateRequired ["+SyncClient.this.updateRequired+"]");
        
        if (SyncClient.this.updateFlag && SyncClient.this.updateRequired)
        {
            return true;
        }
        return false;
    }
    
        
    /**
     * Returns the current status of the update flag 
     * @return  True if there is an update available.
     */
    public boolean isUpdate()
    {
        return updateFlag;
    }
    
    /**
     * Called when user selected Download Update from the menu
     */
    public void doUpdate()
    {
        checkUpdate();
        if (updateFlag)
        {
            synchronized (SyncClient.getEventLock())
            {
                boolean result = askYesNoQuestion(
                                StaticDataHelper.getLanguage("dialog_update"), true);
                if (result)
                {
                    Browser.getDefaultSession().displayPage(updateOTA);
                }
                else
                {
                    showMessage(StaticDataHelper.getLanguage("message_update-location") + ":\n" + updateURL);
                }
            }
        }
    }

    /**
     * Called to warn the user of a large number of deletes
     * @param mask      Bit mask of affected syncsource
     * @returns         Whether or not to continue with the sync
     */
    public boolean warnDelete(int mask)
    {
        Application.getApplication().requestForeground();
        
        Log.info("Prompting user for delete confirmation");
        String warnString = StaticDataHelper.getLanguage("dialog_delete1") + " ";
        int warnMask = 0;
        boolean useComma = false;
        for (int x = 1; x <= SYNC_SUPPORTED; x*=2)
        {
            if ((x & mask) != 0)
            {
                warnMask += x;
                if (useComma)
                {
                    warnString += ", ";
                }

                switch (x)
                {
                    case SYNC_CONTACTS:
                        warnString += StaticDataHelper.getLanguage("sync_contacts");
                        break;
                    case SYNC_CALENDAR:
                        warnString += StaticDataHelper.getLanguage("sync_calendar");
                        break;
                    case SYNC_TASKS:
                        warnString += StaticDataHelper.getLanguage("sync_tasks");
                        break;
                    case SYNC_NOTES:
                        warnString += StaticDataHelper.getLanguage("sync_notes");
                        break;
                }

                useComma = true;
            }
        }
        
        warnString += StaticDataHelper.getLanguage("dialog_delete2");
        
        boolean result = askYesNoQuestion(warnString, false);
        if (result)
        {
            Log.info("Continuing with sync");
        }
        else
        {
            Log.info("User opted to cancel sync - " + warnString);
        }
        
        return result;
    }
    
    /**
     * Check to see if the user should be warned, and if so, warn them.
     * 
     * @param mask
     */
    private int checkDeletes(int mask)
    {
        if (!Configuration.syncContact)
                mask &= ~SYNC_CONTACTS;
        if (!Configuration.syncCalendar)
                mask &= ~SYNC_CALENDAR;
        if (!Configuration.syncTask)
                mask &= ~SYNC_TASKS;
        if (!Configuration.syncNote)
                mask &= ~SYNC_NOTES;
        
        boolean toWarn = false;
        int warnMask   = 0;
        
        for (int x = 1; x <= SYNC_SUPPORTED; x*=2)
        {
            BBPIMSyncSource source = (BBPIMSyncSource)sources.get(new Integer(x));
            if ((x & mask) != 0 && !doRefresh && source != null && source.shouldWarn())
            {
                toWarn = true;
                warnMask += x;
            }
        }
        
        if (toWarn)
        {
            if (!warnDelete(warnMask))
            {
                mask &= ~warnMask;
            }
        }
        
        return mask;
    }
    
    private void runUncompressedSync(int syncMask) {
        // Disable only-in-memory property for compression
        syncConfig.compress = false;

        // Not really necessary, but it makes the code make sense
        compressionRetry = true;

        synchronize(syncMask);
    }

    public boolean isSynchronized(int syncMask)
    {
        boolean ret = true;
        if ((syncMask & SYNC_CONTACTS) != 0)
            if (contactSourceConfig.getNextAnchor() == 0)
                ret = ret && false;
        if ((syncMask & SYNC_CALENDAR) != 0)
            if (calendarSourceConfig.getNextAnchor() == 0)
                ret = ret && false;
        if ((syncMask & SYNC_TASKS) != 0)
            if (taskSourceConfig.getNextAnchor() == 0)
                ret = ret && false;
        if ((syncMask & SYNC_NOTES) != 0)
            if (noteSourceConfig.getNextAnchor() == 0)
                ret = ret && false;
        return ret;
    }

    public void setEnableBlockInvites(boolean enabled)
    {
        BBPIMSyncSource ss = ((BBPIMSyncSource)sources.get(new Integer(SyncClient.SYNC_CALENDAR)));
        if (ss != null)
        {
            ((EventHelper)ss.getHelper()).setIncomingMailListenerStatus(enabled);
        }
    }
    
    public static boolean checkCustom(int flag)
    {
        if (!SyncClient.CUSTOMS_ENABLED)
            return false;
        if ((SyncClient.CUSTOMS & flag) != 0)
            return true;
        return false;
    }

    /**
     * Returns the preferred (default) font for the entire application. Each
     * screen should invoke this method to get the default font. It is no longer
     * possible to set the Font default font because of a bug in OS 4.6 that
     * shows menu item in bold if this is done.
     *
     * @return the preferred font (cannot be null)
     */
    public static Font getPreferredFont() {
        Font font = Font.getDefault();
        try {
            FontFamily ff = null;
            ff = FontFamily.forName("Verdana");
            font = ff.getFont(StaticDataHelper.FONT_STYLE, StaticDataHelper.FONT_SIZE);
        } catch (ClassNotFoundException e) {}

        return font;
    }
        
    
    /**
     * This is the entry point for SyncClient Application. The thread that calls
     * this method (typically the main thread in the application) becomes the
     * event-dispatching thread, which will execute all drawing and
     * event-handling code
     * 
     * @param s[]
     *        An array of strings passed on the command line as parameters
     *        for the <code>java</code> interpreter
     */
    public static void main(String args[]) throws Exception
    {
        if (SyncClient.checkCustom(SyncClient.CUSTOMS_ROLLOVER))
        {
            try
            {
                net.rim.blackberry.api.homescreen.HomeScreen.updateIcon(
                            Bitmap.getBitmapResource("icon.png")
                            );
                net.rim.blackberry.api.homescreen.HomeScreen.setRolloverIcon(
                            Bitmap.getBitmapResource("icon_over.png")
                            );
            }
            catch (Exception e)
            { }
        }

        // Initialize the BlackBerry Record Store for this application
        PluginObjectWrapperHanlder wrapper = new PluginObjectWrapperHanlder();
        BlackberryRecordStore.init(wrapper, "Funambol_PIM_Sync_Client");

        // Start the application
        final SyncClient client = new SyncClient();
        client.enterEventDispatcher();
        // Note that this piece of code is not reachable in normal conditions,
        // as the enterEventDispatcher does not return
    }
    
    class SyncThread extends Thread
    {
        private SyncThread()
        {};
        
        int syncMask;
        
        public SyncThread(int mask)
        {
            this();
            syncMask = mask;
        }
    
        public synchronized void run()
        {
            if (inSynchronize)
            {
                // Notify the user that another sync is in progress
                showScreen(appScreen.getSyncScreen());
                return;
            }

            inSynchronize = true;
            
            boolean wasRefresh = SyncClient.this.doRefresh;

            try
            {
                Log.info("SyncClient.synchronize(int): Starting sync thread - 'inSynchronize' is " + inSynchronize);
                SyncClient.this.syncThreadRun(syncMask);
                compressionRetry = false;
            } 
            catch (CompressedSyncException e)
            {
                if (!compressionRetry)
                {
                    compressionRetry = true;
                }
                else
                {
                    // Should never happen!
                    compressionRetry = false;
                    Log.error("Sync failed with compression error after disabling compression");
                    appScreen.getSyncScreen().displayFailure(
                            StaticDataHelper.getLanguage("sync_failed")
                    );
                }
            }catch (Exception e)
            {    
                compressionRetry = false;
                String message = e.getMessage();
                if (message == null)
                {
                    message = StaticDataHelper.getLanguage("message_no-error");
                }
                appScreen.getSyncScreen().displayFailure(
                            StaticDataHelper.getLanguage("sync_failed") + " - "
                            + message
                    );
            }
            finally
            {
                inSynchronize = false;
                // Retry without compression.
                // This must be in the finally block to avoid race conditions!
                if (compressionRetry)
                {
                    // Only retry because of compression once
                    Log.info("Sync failed because compression failed - Retrying");
                    // Refresh direction is not cleared, so all we need to do is restore the flag
                    SyncClient.this.doRefresh =  wasRefresh; 
                    SyncClient.this.runUncompressedSync(syncMask);
                }

            }
        }

    }
}

