/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import java.util.Date;
import java.util.Hashtable;
import java.util.Calendar;

import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMException;
import javax.microedition.pim.Event;

import net.rim.blackberry.api.mail.event.FolderListener;
import net.rim.blackberry.api.mail.event.StoreListener;
import net.rim.blackberry.api.mail.event.FolderEvent;
import net.rim.blackberry.api.mail.event.StoreEvent;

import net.rim.blackberry.api.mail.SendListener;
import net.rim.blackberry.api.mail.Session;
import net.rim.blackberry.api.mail.Message;
import net.rim.blackberry.api.mail.Store;
import net.rim.blackberry.api.mail.Multipart;
import net.rim.blackberry.api.mail.BodyPart;
import net.rim.blackberry.api.mail.TextBodyPart;
import net.rim.blackberry.api.mail.MessagingException;
import net.rim.device.api.i18n.DateFormat;

import com.funambol.util.Log;
import com.funambol.util.StaticDataHelper;


/**
 * 
 */
public class EventMailListener implements SendListener, FolderListener, StoreListener
{
    public static final String NEW = "";
    public static final String UPD = "Updated: ";
    public static final String DEL = "Cancelled: ";
    
    private Hashtable blockOutgoingSubjects;
    //private Vector blockIncomingTypes;
    //private Vector blockIncomingExtensions;
    private Hashtable blockIncomingEvents;

    private boolean blockOutgoingEnabled;
    private boolean blockIncomingEnabled;
 
    private EventHelper helper = null;
    
    private String inviteReplacementText;
    private String eventDetailsText;
    
    public EventMailListener(EventHelper helper, String repl) 
    {
        this.helper = helper;
        this.inviteReplacementText = repl;
        
        blockOutgoingEnabled    = false;
        blockOutgoingSubjects   = new Hashtable();
        //blockIncomingTypes      = new Vector();
        //blockIncomingExtensions = new Vector();
        blockIncomingEvents     = new Hashtable();
        
        blockIncomingEnabled = false;
        

        
        //blockIncomingType("text/x-vcalendar");
        //blockIncomingType("text/vcalendar");
        //blockIncomingType("text/x-rim-appointment");
        //blockIncomingExtension(".ics");
        
        
        attach();
    }

    public void batchOperation(StoreEvent e)
    {
        //Log.trace("EventMailListener: Batch operation received");
    }
    
    public void messagesAdded(FolderEvent e)
    {
        Message m = e.getMessage();
        
        if (m == null)
        {
            return;
        }
        
        if(blockIncomingEnabled && m.isInbound())
        {
            Log.trace("EventMailListener: Incoming message received - " + m.getSubject());
            
            Object parts = m.getContent();
               
            if (parts == null)
            {
                Log.trace("EventMailListener: Message has no parts - returning");
                return;
            }
               
            /*
            String info = "EventMailListener: \n";
                info += "Subject: " + m.getSubject();
                info += "\n";
                info += "MessageId: " + m.getMessageId();
                info += "\n";
                info += "MessageType: " + m.getMessageType();
                info += "\n";
                info += "ContentType: " + m.getContentType();
                info += "\n";
                info += "BodyText: " + m.getBodyText();
                info += "\n";
                info += "toString: " + m.toString();
                info += "\n";
                info += "size: " + m.getSize();
                info += "\n";
                info += "Flags: " + m.getFlags();
                info += "\n";
                   
            Log.info(info);
            */

            if (parts instanceof Multipart)
            {
                /*
                info = "EventMailListener:\nMultiPart Info : \n";
                info += "contentType: " + mp.getContentType();
                info += "\n";
                info += "toString: " + mp.toString();
                info += "\n";
                info += "Count: "  + mp.getCount();
                info += "\n";
                info += "Parent: " + mp.getParent().toString();
                info += "\n";
                   
                Log.info(info);
                */
                   
                Multipart mp = (Multipart)parts;
                int count = mp.getCount();

                if (count == 0)
                {
                    Log.info("EventMailListener: Multipart Message has no parts - likely an event");

                    if (isIncomingEventBlocked(m.getSubject()))
                    {
                        Log.info("Attempting to delete event from incoming email: " + m.getSubject());
                        if (manageBlockedEvent(m.getSubject()))
                        {
                            clearBlockedIncomingEvents();
                            
                            String text = "\n" + inviteReplacementText + "\n\n" + eventDetailsText;
                            
                            BodyPart txt = new TextBodyPart(mp, text);
                            mp.addBodyPart(txt);
                            try {
                                m.setContent(mp);
                            } catch (MessagingException ex) {
                                Log.info("Unable to add new part");
                            }
                        }
                        else
                        {
                            Log.info("Did not remove event");
                        }
                    }
                }
            }
            Log.debug("messagesAdded complete");
        }
    }
    
    public void messagesRemoved(FolderEvent e)
    {
        //Log.info("--Messages Removed");
    }
    
    
    public boolean sendMessage(Message message) 
    {
        if (blockOutgoingEnabled)
        {
            String subject = message.getSubject();
            boolean block = isOutgoingSubjectBlocked(subject);
            if (block)
            {
                Log.info("Blocking outgoing email for subject: " + subject);
                unblockOutgoingSubject(subject);
                if (!message.getFolder().deleteMessage(message))
                {
                    Log.info("Unable to delete outgoing email: " + subject);
                }
                return false;
            }
        }
        return true;
    }
   


    public void enableBlockOutgoing()
    {
        blockOutgoingEnabled = true;
    }
    
    public void disableBlockOutgoing()
    {
        blockOutgoingEnabled = false;
    }
    
    public void enableBlockIncoming()
    {
        blockIncomingEnabled = true;
    }
    
    public void disableBlockIncoming()
    {
        blockIncomingEnabled = false;
    }

    public void blockOutgoingEvent(String subject, String type)
    {
        if (type.equals(EventMailListener.NEW) || 
            type.equals(EventMailListener.UPD) ||
            type.equals(EventMailListener.DEL)
            )
        {
            blockOutgoingSubject(type + subject);
        }
    }
    
    public void unblockOutgoingEvent(String subject, String type)
    {
        if (type.equals(EventMailListener.NEW) || 
            type.equals(EventMailListener.UPD) ||
            type.equals(EventMailListener.DEL)
            )
        {
            unblockOutgoingSubject(type + subject);
        }
    }
    
    public void unblockAllOutgoingSubjects()
    {
        synchronized(blockOutgoingSubjects)
        {
            blockOutgoingSubjects.clear();
        }
    }
    
    /*
    private boolean isIncomingTypeBlocked(String type)
    {
        if (type == null)
            return false;
        
        boolean ret = false;
        synchronized(blockIncomingTypes)
        {
            if (blockIncomingTypes.contains(type))
            {
                ret = true;
            }
        }
        return ret;
    }
    */
    /*
    private boolean isIncomingExtensionBlocked(String filename)
    {
        if (filename == null)
            return false;
        
        boolean ret = false;
        synchronized(blockIncomingExtensions)
        {
            int index = filename.indexOf(".");
            if (index > 0)
            {
                String ext = filename.substring(index); 
                if (blockIncomingExtensions.contains(ext))
                {
                    ret = true;
                }
            }
        }
        return ret;
    }
    */
    
    private boolean isOutgoingSubjectBlocked(String subject)
    {
        if (subject == null)
            return false;
        
        boolean ret = false;
        synchronized(blockOutgoingSubjects)
        {
            if (blockOutgoingSubjects.containsKey(subject))
            {
                ret = true;
            }
        }
        return ret;
    }
    
    
    private boolean isIncomingEventBlocked(String subject)
    {
        Log.info("isIncomingEventBlocked?");
        if (subject == null)
        {
            Log.info("isIncomingEventBlocked: null subject");
            return false;
        }
        
        boolean ret = false;
        synchronized(blockIncomingEvents)
        {
            java.util.Enumeration keys = blockIncomingEvents.keys();
            
            while (keys.hasMoreElements())
            {
                String key = (String)keys.nextElement();
                Log.info("Comparing " + key + " with subject " + subject);
                if (subject.endsWith(key));
                {
                    ret = true;
                    break;
                }
            }
            
            Log.info("isIncomingEventBlocked: No more subjects to check");
        }
        return ret;
    }
    
    private boolean manageBlockedEvent(String subject)
    {
        Log.info("Managing blocked event");
        boolean ret = false;
        synchronized(blockIncomingEvents)
        {
            PIMItem item = null;
            java.util.Enumeration keys = blockIncomingEvents.keys();
            while (keys.hasMoreElements())
            {
                String key = (String)keys.nextElement();
                if (subject.endsWith(key));
                {
                    item = (PIMItem) blockIncomingEvents.get(key);
                    break;
                }
            }
            
            eventDetailsText = new String("");
            
            if (item != null)
            {
            	Calendar temp;
            	if (helper.isSupportedField(Event.SUMMARY)) {
                    try {
                    	eventDetailsText += "Subject: " + item.getString(Event.SUMMARY, 0) + "\n";
                    } catch (final Exception e){}
            	}
            	if (helper.isSupportedField(Event.LOCATION)) {
                    try {
                		eventDetailsText += "Location: " + item.getString(Event.LOCATION, 0) + "\n";
                    } catch (final Exception e){}
            	}
            	if ((temp = helper.getFieldDate(item, Event.START, false, false, false)) != null) {
                    try {
                		eventDetailsText += "Start: " + 
                        DateFormat.getInstance(DateFormat.DATE_MEDIUM | DateFormat.TIME_SHORT).format(
                        		new Date(PIMItemHelper.convertGMTToLocal(PIMItemHelper.getTimestamp(temp)))
                        		)
                        + "\n"; 
                    } catch (final Exception e){}
            	}
            	if ((temp = helper.getFieldDate(item, Event.END  , false, false, false)) != null) {
                    try {
                		eventDetailsText += "End: " + 
                        DateFormat.getInstance(DateFormat.DATE_MEDIUM | DateFormat.TIME_SHORT).format(
                        		new Date(PIMItemHelper.convertGMTToLocal(PIMItemHelper.getTimestamp(temp)))
                        		)
                        + "\n";
                    } catch (final Exception e){}
            	}
                if (helper.isSupportedField(Event.NOTE)) {
                    try {
                		eventDetailsText += "Notes: " + item.getString(Event.NOTE, 0) + "\n";
                    } catch (final Exception e){}
            	}
                if (((Event)item).getRepeat() != null)
                	eventDetailsText += "This event is recurring." + "\n";
            	
                Log.info("Attemptng to delete event");
                try {
                    helper.deleteItem(item);
                    ret = true;
                } catch (PIMException e) {
                    String msg = helper.handlePIMException(e);
                    Log.error("Error removing event from email: " + msg);
                }
            }
        }
        return ret;
    }
    
    private void clearBlockedIncomingEvents()
    {
        synchronized(blockIncomingEvents)
        {
            Log.info("Clearing blocked incoming event list");
            blockIncomingEvents.clear();
        }
    }
    
    
    private void blockOutgoingSubject(String subject)
    {
        synchronized(blockOutgoingSubjects)
        {
            Log.debug("Registering subject to be blocked: " + subject);
            if (blockOutgoingSubjects.containsKey(subject))
            {
                Integer val = (Integer)blockOutgoingSubjects.get(subject);
                val = new Integer(val.intValue()+ 1);
                blockOutgoingSubjects.put(subject,val);
            }
            else
            {
                blockOutgoingSubjects.put(subject, new Integer(1));
            }
        }
    }
    
    private void unblockOutgoingSubject(String subject)
    {
        synchronized(blockOutgoingSubjects)
        {
            Log.debug("Unregistering subject to be blocked: " + subject);
            if (blockOutgoingSubjects.containsKey(subject))
            {
                Integer val = (Integer)blockOutgoingSubjects.get(subject);
                if (val.intValue()-1 <= 0)
                {
                    blockOutgoingSubjects.remove(subject);    
                }
                else
                {
                    blockOutgoingSubjects.put(subject, new Integer(val.intValue()-1));
                }
            }
        }
    }
    
    public void blockIncomingEvent(String subject, PIMItem item)
    {
        synchronized (blockIncomingEvents)
        {
            clearBlockedIncomingEvents();
            Log.debug("Registering next subject to be blocked: " + subject);
            blockIncomingEvents.put(subject, item);
        }
    }
    
    public void unblockIncomingEvent(String subject)
    {
        synchronized (blockIncomingEvents)
        {
            Log.debug("Registering next subject to be blocked: " + subject);
            if (!blockIncomingEvents.contains(subject))
                blockIncomingEvents.remove(subject);
        }
    }
    /*
    private void blockIncomingType(String type)
    {
        synchronized(blockIncomingTypes)
        {
            Log.debug("Registering type to be blocked: " + type);
            if (!blockIncomingTypes.contains(type))
                blockIncomingTypes.addElement(type);
        }
    }
    */
    
    /*
    private void unblockIncomingType(String type)
    {
        synchronized(blockIncomingTypes)
        {
            Log.debug("Unregistering type to be blocked: " + type);
            blockIncomingTypes.removeElement(type);
        }
    }
    */
    /*
    private void blockIncomingExtension(String ext)
    {
        synchronized(blockIncomingTypes)
        {
            Log.debug("Registering extension to be blocked: " + ext);
            if (!blockIncomingExtensions.contains(ext))
                blockIncomingExtensions.addElement(ext);
        }
    }
    */
    /*
    private void unblockIncomingExtension(String ext)
    {
        synchronized(blockIncomingTypes)
        {
            Log.debug("Unregistering extension to be blocked: " + ext);
            blockIncomingExtensions.removeElement(ext);
        }
    }
    */
    
    public void attach()
    {
        Store mailStore = null;
        /*try {
           mailStore = Session.waitForDefaultSession().getStore();
        } catch (NoSuchServiceException e) {
            Log.error("Unable to get mail store: " + e.toString());
        }*/
        Session sess = Session.getDefaultInstance();
        if (sess != null)
        {
        	mailStore = sess.getStore();
        }

        if (mailStore == null)
        {
            Log.error("Unable to get mail store");
            return;
        }
        
        // Remove the listeners in case we're attaching for a second time
        try {
        	mailStore.removeSendListener(this);
        } catch (Exception e) {}
        try {
            mailStore.removeFolderListener(this);
        } catch (Exception e) {}
        try {
        	mailStore.removeStoreListener(this);
        } catch (Exception e) {}

        
        mailStore.addSendListener(this);
        mailStore.addFolderListener(this);
        mailStore.addStoreListener(this);
        
        Log.info("Mail listener attached");
    }
} 
