/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.io.ByteArrayInputStream;
import java.io.InputStreamReader;
import java.io.ByteArrayOutputStream;
import java.io.UnsupportedEncodingException;

import javax.microedition.pim.PIM;
import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;
import javax.microedition.pim.PIMException;

import javax.microedition.pim.Contact;
import javax.microedition.pim.ContactList;

import net.rim.device.api.io.Base64InputStream;
import net.rim.device.api.system.EncodedImage;

import com.funambol.common.pim.*;
import com.funambol.common.pim.vcard.*;

import com.funambol.util.StaticDataHelper;
import com.funambol.util.XmlUtil;
import com.funambol.util.Log;

/**
 * This class is the helper for BB vCard sync source. It is responsible for
 * adding/updating changes from the server and reading items from the local store.
 * The class also provides utility methods used by the BBPimSyncSource.
 * VCard parsing and formatting is carried out by classes implemented in j2me
 * apis.
 */

public class VCardContactHelper extends ContactHelper
{

    private static final int[] SUPPORTED_FIELDS =
    {
        Contact.NAME
        ,Contact.EMAIL
        ,Contact.NOTE
        ,Contact.TITLE
        ,Contact.ORG
        ,Contact.URL
        ,Contact.TEL
        ,Contact.ADDR
        ,Contact.PHOTO
    };


    /**
     * The initialization constructor
     * @param ContactList A BlackBerry contacts list
     * @param Contact A BlackBerry contact
     * @param boolean To modify or not to modify: that is the question...
     */
    public VCardContactHelper(PIMList list)
    {
        super(list);
    }

    /**
     * This class extends the default VCardFormatter to reimplement the method
     * that autodetects the mime type of a picture. Standard j2me APIs do not
     * allow for such a feature, so we rely on RIM specific APIs to carry out
     * the task.
     */
    private class BBVCardFormatter extends VCardFormatter {

        public BBVCardFormatter() {
            super();
        }

        public BBVCardFormatter(String defaultCharset) {
            super(defaultCharset);
        }

        // Is this really necessary? How does the BB stores photos? If they are
        // always converted to jpeg we are safe, otherwise we need this code
        // to detect the type
        protected String getPhotoType(byte[] photo) {
            try {
                byte[] photoDecoded = Base64InputStream.decode(photo, 0, photo.length);
                EncodedImage image = EncodedImage.createEncodedImage(photoDecoded, 0, -1);

                String imageType = image.getMIMEType();
                int index = imageType.indexOf("/");
                String type = imageType.substring(index+1);

                return type;
            } catch (Exception e) {
                Log.error("[BBVCardFormatter] Cannot detect image type");
                return super.getPhotoType(photo);
            }
        }
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#toString(PIMList pimlist,PIMItem item)
     */
    public String itemToString(PIMItem item) throws PIMException
    {
        //Log.info("toString - starting " + Calendar.getInstance().getTime().toString());
        if (item instanceof Contact)
        {
            // should we remove this line after cleaning out the code
            ContactList list = (ContactList)getPIMList();
            
            Contact contact = (Contact)item;

            ByteArrayOutputStream ostream = new ByteArrayOutputStream();
            VCardFormatter formatter = new BBVCardFormatter("UTF-8");
            formatter.format(contact, ostream, true);
            String vCard = ostream.toString();
            return vCard;
        }
        return null;
    }

    //----------------------------------------------------------
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#createPIMItem(PIMList pimlist)
     */
    public PIMItem createPIMItem(PIMList pimlist) throws PIMException
    {
        if (pimlist instanceof ContactList)
        {
            return ((ContactList)pimlist).createContact();
        }
        return null;
    }

    //----------------------------------------------------------
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#setPIMProperties(PIMList pimlist, PIMItem item,String pimString)
     */
    public void fillItem(PIMItem item, String pimString) throws PIMException
    {
        Log.debug("pimString ["+pimString+"]");
        PIMList pimlist = getPIMList();
        if (pimlist instanceof ContactList)
        {
            ContactList list = (ContactList)pimlist;
            ByteArrayInputStream is = new ByteArrayInputStream(pimString.getBytes());
            PIM pim = PIM.getInstance();

            try {
                Contact newContact = (Contact)item;
                ContactParserListener lis = new ContactParserListener(newContact);
                VCardSyntaxParser parser = new VCardSyntaxParser(is);
                parser.setListener(lis);
                parser.parse();
            } catch (Exception pe) {
                Log.debug("Exception while parsing the item " + pe.toString());
                throw new PIMException(pe.toString());
            }
        }
    }
    
    //----------------------------------------------------------
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#setPIMProperties(PIMList pimlist, PIMItem item,String pimString)
     */
    public void updatePimItem(PIMList pimlist, PIMItem item,String pimString) throws PIMException
    {
        Log.debug("pimString ["+pimString+"]");
        if (pimlist instanceof ContactList)
        {
            ContactList list = (ContactList)pimlist;
            ByteArrayInputStream is = new ByteArrayInputStream(pimString.getBytes());
            PIM pim = PIM.getInstance();

            try {
                Contact newContact = (Contact)item;
                ContactParserListener lis = new ContactParserListener(newContact);
                VCardSyntaxParser parser = new VCardSyntaxParser(is);
                parser.setListener(lis);
                parser.parse();
            } catch (ParseException pe) {
                Log.debug("Exception while adding the item " + pe.toString());
                throw new PIMException(pe.toString());
            }
        }
    }
    
    //----------------------------------------------------------
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#find_and_deletePIMItem(PIMList pimlist, String itemKey, boolean deleteFlag)
     */
    public PIMItem findPIMItem(PIMList pimlist, String itemKey, boolean deleteFlag) throws PIMException
    {
        if (pimlist instanceof ContactList){
            ContactList _pimlist = (ContactList)pimlist;
            PIMItem item = null;
            Enumeration items = _pimlist.items();
            while(items.hasMoreElements())
            {
                item = (PIMItem)items.nextElement();
                if (itemKey.equals(this.getUID(item)))
                {
                    if (deleteFlag)
                    {
                        _pimlist.removeContact((Contact)item);
                    }
                    items = null;
                    _pimlist = null;
                    return item;
                }
            }
            item = null;
            items = null;
            _pimlist = null;
        }
        return null;
    }
    //----------------------------------------------------------
    public Hashtable findPIMItems(PIMList pimlist, Vector itemKeys, boolean deleteFlag) throws PIMException
    {
        Hashtable ret = new Hashtable();
        if (pimlist instanceof ContactList)
        {
            ContactList _pimlist = (ContactList)pimlist;
            PIMItem item = null;
            Enumeration items = _pimlist.items();
            while(items.hasMoreElements())
            {
                item = (PIMItem)items.nextElement();
                String uid = this.getUID(item);
                if (itemKeys.contains(uid))
                {
                    if (deleteFlag)
                    {
                        _pimlist.removeContact((Contact)item);
                    }
                    ret.put(uid,item);
                    itemKeys.removeElement(uid);
                }
            }
            item = null;
            items = null;
            _pimlist = null;
        }
        return ret;
    }


    //----------------------------------------------------------
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#clearPIMItem(PIMItem _item)
     */
    public void clearItem(PIMItem item) throws PIMException
    {
        PIMItemHelper.clearPIMItem(item,Contact.UID, SUPPORTED_FIELDS);
    }

    public void copyUnsupportedFields(PIMItem origItem, PIMItem item) throws PIMException
    {
        copyUnsupportedFields(origItem, item, Contact.UID, SUPPORTED_FIELDS);
        // We shall copy also unsupported attributes of supported fields
        Contact source = (Contact)origItem;
        Contact target = (Contact)item;

        int[] attributes = source.getPIMList().getSupportedAttributes(Contact.TEL);

        for (int i = 0; i < source.countValues(Contact.TEL); i++)
        {
            final String value = source.getString(Contact.TEL, i);
            if (value != null) {
                final int attribute = source.getAttributes(Contact.TEL, i);

                if (attribute != Contact.ATTR_FAX &&
                    attribute != Contact.ATTR_PAGER &&
                    attribute != Contact.ATTR_MOBILE &&
                    attribute != Contact.ATTR_WORK &&
                    attribute != Contact.ATTR_HOME &&
                    attribute != Contact.ATTR_OTHER)
                {

                    Log.trace("Preserving attribute: " + attribute);
                    target.addString(Contact.TEL, attribute, value);
                }
            }
        }
    }


    //----------------------------------------------------------
    public void deleteAllPIMItems(PIMList pimlist) throws PIMException
    {
        if (pimlist instanceof ContactList)
        {
            ContactList _pimlist = (ContactList)pimlist;
            PIMItem item = null;
            Enumeration items = _pimlist.items();
            while(items.hasMoreElements())
            {
                item = (PIMItem)items.nextElement();
                if (this.isInSyncRange(pimlist,item))
                {
                    _pimlist.removeContact((Contact)item);
                }
            }
            item = null;
            items = null;
            _pimlist = null;
        }
    }
    //----------------------------------------------------------
    public boolean isInSyncRange(PIMList pimlist, PIMItem _item) throws PIMException
    {
        return true;
    }

    public void setPIMProperties(PIMList pimlist, PIMItem item,String pimString) throws PIMException
    {
    }

    //----------------------------------------------------------
} 

