/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
package com.funambol.blackberry.ui;

import java.util.Date;

import net.rim.device.api.i18n.DateFormat;
import net.rim.device.api.system.Bitmap;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.BitmapField;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.Font;

import net.rim.device.api.ui.container.HorizontalFieldManager;

import com.funambol.util.StaticDataHelper;
import com.funambol.blackberry.SyncClient;
import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.BBVersion;
import com.funambol.blackberry.ui.CenteredManager;
import com.funambol.util.ConnectionManager;
import com.funambol.util.ConnectionListener;
import com.funambol.util.Log;

/**
 * AppScreen is an inner class that creates the home screen for SyncClient on
 * the BlackBerry. It extends MainScreen.
 * 
 */
public class AppScreen extends MainScreen implements ConnectionListener
{
    final public static String          IMAGE_LOGO      = "logo.png";

    private final SyncClient            app;
    
    /**
     * The menu
     */
    private final MenuItem              syncMenu            = new MenuItem
        (StaticDataHelper.getLanguage("menu_sync"), 20000, 10)
        { public void run() {
            SyncClient.syncType = SyncClient.MANUAL_SYNC; 
            app.synchronize();
        } };
    private final MenuItem          refreshMenu             = new MenuItem
        (StaticDataHelper.getLanguage("menu_refresh"), 20000, 10)
        { public void run() {
            if (!
                app.promptNext(StaticDataHelper.getLanguage("dialog_refresh_warn"))
                )
            {
                return;
            }
            
            int direction = showRefreshDirectionDialog();
            if (direction > 0)
            {
                int mask = showRefreshTypeDialog(direction);
                if (mask > 0)
                {
                    SyncClient.showScreen(syncScreen);
                    app.refresh(mask, direction);
                }
            }
        } };
    private final MenuItem              statusMenu          = new MenuItem
        (StaticDataHelper.getLanguage("menu_status"), 20000, 10)
        { public void run() {
            SyncClient.showScreen(syncScreen);
        } };
    private final MenuItem              errorMenu           = new MenuItem
        (StaticDataHelper.getLanguage("menu_error"), 20000, 10)
        { public void run() {
            showErrors();
        } };
    private final MenuItem              configMenu          = new MenuItem
        (StaticDataHelper.getLanguage("menu_config"), 20000, 10)
        { public void run() {
            configScreen = new ConfigurationScreen(app);
            SyncClient.showScreen(configScreen);
        } };
    private final MenuItem             helpMenu             = new MenuItem
        (StaticDataHelper.getLanguage("menu_help"), 20000, 10)
        { public void run() {
            helpScreen = new HelpScreen(app);
            SyncClient.showScreen(helpScreen);
        } };
    private final MenuItem             updateMenu           = new MenuItem
        (StaticDataHelper.getLanguage("menu_update"), 20000, 10)
        { public void run() {
            app.doUpdate();
        } };
    private final MenuItem             quitMenu             = new MenuItem
        (StaticDataHelper.getLanguage("menu_quit"), 20000, 10)
        { public void run() {
            app.minimize();
        } };        

    private ConfigurationScreen        configScreen;
    private HelpScreen                 helpScreen;
    private final SyncScreen           syncScreen;
    
    private LabelField                 lastSync = null;

    /**
     * 
     * @return    The mask for the selected category(ies).
     */ 
    private int showRefreshTypeDialog(int direction)
    {
        String [] opt = new String [6];
        int [] val = new int[6];
        int index = 0;
        
        if (Configuration.syncContact)
        {
            opt[index] = StaticDataHelper.getLanguage("sync_contacts");
            val[index] = SyncClient.SYNC_CONTACTS;
            index++;
        }
        
        if (Configuration.syncCalendar)
        {
            opt[index] = StaticDataHelper.getLanguage("sync_calendar");
            val[index] = SyncClient.SYNC_CALENDAR;
            index++;
        }
        
        if (Configuration.syncTask)
        {
            opt[index] = StaticDataHelper.getLanguage("sync_tasks");
            val[index] = SyncClient.SYNC_TASKS;
            index++;
        }
        
        if (Configuration.syncNote)
        {
            opt[index] = StaticDataHelper.getLanguage("sync_notes");
            val[index] = SyncClient.SYNC_NOTES;
            index++;
        }
        
        if (index > 1)
        {
            opt[index] = StaticDataHelper.getLanguage("sync_all");
            val[index]  = SyncClient.SYNC_ALL;
            index++;
        }
        
        opt[index] = StaticDataHelper.getLanguage("sync_cancel");
        val[index] = -1;
        index++;
               
        String [] options = new String [index];
        int [] values = new int [index];
        
        for (int x = 0 ; x < index; x++)
        {
            options[x] = opt[x];
            values[x] = val[x];
        }
                
        return Dialog.ask(
            StaticDataHelper.getLanguage("dialog_refresh") + " " + 
            (direction == SyncClient.REFRESH_FROM_SERVER ? 
                    StaticDataHelper.getLanguage("dialog_refresh_from") :
                    StaticDataHelper.getLanguage("dialog_refresh_to")
            ),
            options,
            values,
            -1
        );
    }

    /**
     * 
     * @return    The selected direction.
     */ 
    private int showRefreshDirectionDialog()
    {
        String [] opt = new String [3];
        int [] val = new int[3];

        opt[0] = StaticDataHelper.getLanguage("dialog_refresh_from");
        val[0] = SyncClient.REFRESH_FROM_SERVER;
        
        opt[1] = StaticDataHelper.getLanguage("dialog_refresh_to");
        val[1] = SyncClient.REFRESH_TO_SERVER;
        
        opt[2] = StaticDataHelper.getLanguage("sync_cancel");
        val[2] = -1;
                               
        return Dialog.ask(
            StaticDataHelper.getLanguage("dialog_refresh_which") + "\n" +
            StaticDataHelper.getLanguage("dialog_refresh_warn2"),
            opt,
            val,
            -1
        );
    }
    
    /**
     * Get the Sync Screen
     * @return
     */
    public SyncScreen getSyncScreen()
    {
        return syncScreen;
    }
    
    /**
     * Set the displayed last sync time
     * @param time
     */
    public void setLastSync(long time)
    {
        synchronized(SyncClient.getEventLock())
        {
            DateFormat df = DateFormat.getInstance(DateFormat.DATE_MEDIUM | DateFormat.TIME_SHORT);
            String msg = df.formatLocal(time);
            lastSync.setText(msg);
        }
    }
    
    /**
     * Creates the home screen with all the UI components on it
     * @param app
     */
    public AppScreen(SyncClient app)
    {
        // The entire layout is completely hardcoded today. Can we make it
        // flexible? Are there layout managers or shall we create a simple one?

        // Set the screen font
        this.setFont(SyncClient.getPreferredFont());

        this.app = app;
        this.syncScreen   = new SyncScreen(app);
       
        CenteredManager cm = new CenteredManager();
        
        LabelField temp;
        LabelField title; 
        //Bitmap icon = null;
        //BitmapField iconField = null;
        //VerticalFieldManager vfm;
        HorizontalFieldManager hfm; 
        
        if (SyncClient.checkCustom(SyncClient.CUSTOMS_TITLE_HAS_VERSION))
        {
            title= new LabelField(StaticDataHelper.getLanguage("home_title") + " " + BBVersion.VERSION);
        }else{
            title= new LabelField(StaticDataHelper.getLanguage("home_title"));
        }

        title.setFont(getFont().derive(
                StaticDataHelper.TITLE_FONT_STYLE,
                StaticDataHelper.TITLE_FONT_SIZE)
            );
        title.setPadding(
                StaticDataHelper.TITLE_PADDING,
                StaticDataHelper.TITLE_PADDING,
                StaticDataHelper.TITLE_PADDING,
                StaticDataHelper.TITLE_PADDING
            );
        setTitle(title);

        cm.add(new LabelField(""));

        Bitmap logo = Bitmap.getBitmapResource(IMAGE_LOGO);
        BitmapField image = new BitmapField(logo, Field.FIELD_HCENTER);
        image.setMargin(0, 0,0,0);
        image.setSpace(-4, -4);
        /////
        cm.add(image);
        
        temp = new LabelField(StaticDataHelper.getLanguage("home_welcome"),
                        Field.FIELD_HCENTER);
        temp.setFont(getFont().derive(Font.BOLD));
        cm.add(temp);
        temp = new LabelField(StaticDataHelper.getLanguage("home_welcome_second"),
                        Field.FIELD_HCENTER);
        temp.setFont(getFont().derive(Font.BOLD));
        cm.add(temp);
        temp = new LabelField(StaticDataHelper.getLanguage("home_click-menu"),
                        Field.FIELD_HCENTER);
        cm.add(temp);
        cm.add(new LabelField(""));
        
        hfm = new HorizontalFieldManager(Field.FIELD_HCENTER);
        temp = new LabelField(StaticDataHelper.getLanguage("home_last-sync") + " ",
                        Field.FIELD_VCENTER);
        hfm.add(temp);
        lastSync = new LabelField("N/A", Field.FIELD_VCENTER);
        hfm.add(lastSync);
        cm.add(hfm);
 
        cm.add(new LabelField(""));// Need a small offset to make it look nice
        cm.add(new LabelField(""));

        add(cm);
    }
    
    /**
     * Called when the user selected the Error Log option
     */
    public void showErrors()
    {
        //SyncClient.showMessage(StaticDataHelper.getErrors().getErrors());
    }
    
    /**
     * Builds the menu
     * 
     * @param Menu
     *        Menu to which menuitems are to be added
     * @param int
     *        instance
     */
    public void makeMenu(Menu menu, int instance)
    {
        if (!app.inSync())
        {
            menu.add(syncMenu);             
            menu.add(refreshMenu);
        }
        menu.add(statusMenu);
        /*
        if (StaticDataHelper.getErrors().numErrors() != 0)
        {
            menu.add(errorMenu);
        }
        */
        if (!app.inSync())
        {
            menu.add(configMenu);
        }
        if (SyncClient.checkCustom(SyncClient.CUSTOMS_UPDATE))
        {
            if (app.isUpdate())
            {
                menu.add(updateMenu);
            }
        }
        menu.add(helpMenu);
        menu.add(quitMenu);
    }
    
    /**
     * This method is invoked on user hitting the escape button when in home
     * screen. It only minimizes the program
     */
    public boolean onClose()
    {
        this.getApplication().requestBackground();
        return true;
    }
    
    public boolean isConnectionConfigurationAllowed(final String arg0) {
        try {

            final QuestionScreen qs = new QuestionScreen(arg0);
            System.out.println("before syncronized code");

            synchronized (QuestionScreen.lock) {
                System.out.println("pushing screen");

                app.invokeLater(new Runnable() {

                    public void run() {
                        app.pushScreen(qs);
                    }
                });
                // app.pushScreen(qs);

                System.out.println("lock.wait");
                QuestionScreen.lock.wait();

                System.out.println("response has been given");
                if (QuestionScreen.response == QuestionScreen.CONFIRM) {
                    System.out.println("returning true");
                    return true;
                } else {
                    System.out.println("returning false");

                    return false;
                }
            }

        } catch (InterruptedException ex) {
            ex.printStackTrace();
            System.out.println("interruped exception");
        }
        System.out.println("returning false, but don't know why I'm here!");
        return false;
    }
    
    public void connectionOpened() {
        Log.debug("[AppScreen]Connection opened");
    }

    public void requestWritten() {
        Log.debug("[AppScreen]Request written");
    }

    public void responseReceived() {
        Log.debug("[AppScreen]Response received");
    }

    public void connectionClosed() {
        Log.debug("[AppScreen]Connection closed");
    }

    public void connectionConfigurationChanged() {
        Log.debug("[AppScreen]Configuration changed");
    }
}
