/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui;

import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.BasicEditField;
import net.rim.device.api.ui.component.CheckboxField;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.component.ObjectChoiceField;
import net.rim.device.api.ui.component.NumericChoiceField;
import net.rim.device.api.ui.component.PasswordEditField;
import net.rim.device.api.ui.component.SeparatorField;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Font;

import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.Customization;
import com.funambol.util.StaticDataHelper;
import com.funambol.blackberry.SyncClient;
import com.funambol.util.Log;
import com.funambol.blackberry.Customization;

/**
 * Is an class that creates the configuration screen for
 * configuring parameters required for the SyncClient
 */
public class ConfigurationScreen extends MainScreen
{
    /**
     * This class is only used if CUSTOM_COMPANY_DIRECTORY is enabled.
     * 
     */
    class CDListener implements FieldChangeListener
    {
        public void fieldChanged(Field field, int context)
        {
            CheckboxField cd = (CheckboxField)field; 
            
            String curr = ConfigurationScreen.this.contactSourceUriField.getText();
            if (cd.getChecked())
            {
                if (curr.endsWith("-shared"))
                {
                    // Do nothing
                }
                else
                {
                    curr = curr + "-shared";
                }
            }
            else
            {
                if (curr.endsWith("-shared"))
                {
                    curr = curr.substring(0, curr.length() - "-shared".length());
                    SyncClient.showMessage(StaticDataHelper.getLanguage("custom_message_remove-cd"));
                }
                else
                {
                    // Do nothing
                }
            }
            ConfigurationScreen.this.contactSourceUriField.setText(curr);
        }
    }
    
    
    
    private final SyncClient        app;
        
    private BasicEditField          serverUriField          = null;

    private BasicEditField          usernameField           = null;
    private PasswordEditField       passwordField           = null;

    private BasicEditField          calendarSourceUriField  = null;
    private BasicEditField          contactSourceUriField   = null;
    private BasicEditField          taskSourceUriField      = null;
    private BasicEditField          noteSourceUriField      = null;

    private CheckboxField           enableSyncCalendarField = null;
    private CheckboxField           enableSyncContactField  = null;
    private CheckboxField           enableSyncNoteField     = null;
    private CheckboxField           enableSyncTaskField     = null;

    private CheckboxField           enablePollingPIMField   = null;    
    private NumericChoiceField      pollingPIMIntervalField = null;
    
    private NumericChoiceField      rangePastField            = null;
    private NumericChoiceField      rangeFutureField        = null;

    private ObjectChoiceField       syncTypeField           = null;
    private ObjectChoiceField       logLevelField           = null;

    private boolean hasChanges                              = true; //TODO set dynamically
    
    /**
     * This creates a MenuItem containing "Save" option. on selecting the save
     * option the configuration details are saved.
     */
    private final MenuItem          saveMenu      = new MenuItem(
                StaticDataHelper.getLanguage("menu_save"), 20000, 10)
                { public void run() {
                    if (saveConfiguration())
                    {
                        app.hideScreen(ConfigurationScreen.this);
                    }
                }};
                
    private final MenuItem          backMenu     = new MenuItem(
                StaticDataHelper.getLanguage("menu_back"), 20000, 10)
                { public void run() {
                        onClose();
                }};
                
    /**
     * The constructor here creates the Configuration screen with all the UI
     * components on it and sets default input values
     */
    public ConfigurationScreen(SyncClient app)
    {
        super(Field.USE_ALL_WIDTH |
              Field.USE_ALL_HEIGHT |
              Field.FIELD_LEFT |
              Field.FIELD_TOP);
        
        this.app = app;
        this.setFont(SyncClient.getPreferredFont());

        LabelField title = new LabelField(StaticDataHelper.getLanguage("conf_title"));
        title.setFont(getFont().derive(
                        StaticDataHelper.TITLE_FONT_STYLE,
                        StaticDataHelper.TITLE_FONT_SIZE)
                        );
        title.setPadding(
                StaticDataHelper.TITLE_PADDING,
                StaticDataHelper.TITLE_PADDING,
                StaticDataHelper.TITLE_PADDING,
                StaticDataHelper.TITLE_PADDING);
        setTitle(title);

        LabelField serverUriLabel = null;
        if (Customization.getInstance().syncUriEditable()) {
            serverUriLabel = new LabelField(StaticDataHelper.getLanguage("conf_server-url"));
            serverUriLabel.setFont(getFont().derive(Font.BOLD));
            serverUriField = new BasicEditField("", Configuration.syncUrl);
        }
        
        LabelField usernameLabel = new LabelField(StaticDataHelper.getLanguage("conf_user"));
        usernameLabel.setFont(getFont().derive(Font.BOLD));
        usernameField = new BasicEditField("", Configuration.username);
        
        LabelField passwordLabel = new LabelField(StaticDataHelper.getLanguage("conf_password"));
        passwordLabel.setFont(getFont().derive(Font.BOLD));
        passwordField = new PasswordEditField("", Configuration.password);

        LabelField enableSyncContactLabel = null;
        if (Customization.getInstance().contactEnabled()) {
            enableSyncContactLabel = new LabelField(StaticDataHelper.getLanguage("conf_enable-sync-contact"));
            enableSyncContactLabel.setFont(getFont().derive(Font.BOLD));
            enableSyncContactField = new CheckboxField(
                    "",
                    Configuration.syncContact
                    );
            contactSourceUriField = new BasicEditField(
                    StaticDataHelper.getLanguage("conf_remote-uri") + " ",
                    Configuration.contactSourceUri
                    );
        }
        
        CheckboxField cdfield = null;
        if (SyncClient.checkCustom(SyncClient.CUSTOMS_COMPANY_DIRECTORY))
        {
            cdfield = new CheckboxField(
                    StaticDataHelper.getLanguage("custom_config_cd"), Configuration.contactSourceUri.endsWith("-shared"));
            FieldChangeListener x = new CDListener();
            cdfield.setChangeListener(x);
        }
        
        LabelField enableSyncCalendarLabel = null;
        if (Customization.getInstance().calendarEnabled()) {
            enableSyncCalendarLabel = new LabelField(StaticDataHelper.getLanguage("conf_enable-sync-calendar"));
            enableSyncCalendarLabel.setFont(getFont().derive(Font.BOLD));
            enableSyncCalendarField = new CheckboxField(
                    "",
                    Configuration.syncCalendar
                    );
            calendarSourceUriField = new BasicEditField(
                    StaticDataHelper.getLanguage("conf_remote-uri") + " ",
                    Configuration.calendarSourceUri
                    );
        }
            
        LabelField enableSyncTaskLabel = null;
        if (Customization.getInstance().taskEnabled()) {
            enableSyncTaskLabel = new LabelField(StaticDataHelper.getLanguage("conf_enable-sync-task"));
            enableSyncTaskLabel.setFont(getFont().derive(Font.BOLD)); 
            enableSyncTaskField = new CheckboxField(
                    "",
                    Configuration.syncTask
                );
            taskSourceUriField = new BasicEditField(
                    StaticDataHelper.getLanguage("conf_remote-uri") + " ",
                    Configuration.taskSourceUri
                );
        }
                
        LabelField enableSyncNoteLabel = null;
        if (Customization.getInstance().noteEnabled()) {
            enableSyncNoteLabel = new LabelField(StaticDataHelper.getLanguage("conf_enable-sync-note"));
            enableSyncNoteLabel.setFont(getFont().derive(Font.BOLD));
            enableSyncNoteField = new CheckboxField(
                    "",
                    Configuration.syncNote
                );
            noteSourceUriField = new BasicEditField(
                    StaticDataHelper.getLanguage("conf_remote-uri") + " ",
                    Configuration.noteSourceUri
                );
        }

        LabelField enablePollingPIMLabel = new LabelField(
                StaticDataHelper.getLanguage("conf_enable-polling-pim")
                );
        enablePollingPIMLabel.setFont(getFont().derive(Font.BOLD));
        enablePollingPIMField = new CheckboxField(
                "", Configuration.enablePollingPIM);
      
        int min = 30;
        int interval = 30;
        int max = 240;
        pollingPIMIntervalField = new NumericChoiceField(
                StaticDataHelper.getLanguage("conf_polling-pim-duration") + "  ",
            min,// begin
            max,// end
            interval,// increment
            (Configuration.pollingPIMInterval >= min ?
                        (Configuration.pollingPIMInterval - min) / interval
                        : 0)// initial index (not value!)
                );
        
        /*
         * TWO_WAY = 0;
         * ONE_WAY_TO_SERVER = 1;
         * ONE_WAY_TO_CLIENT = 2;
         */
        String [] choices = new String []
        {
                StaticDataHelper.getLanguage("conf_sync-type-two-way"),
                StaticDataHelper.getLanguage("conf_sync-type-one-way-to-server"),
                StaticDataHelper.getLanguage("conf_sync-type-one-way-to-client"),
        };
        

        LabelField syncTypeLabel = new LabelField(
                        StaticDataHelper.getLanguage("conf_sync-type")
                        );
        syncTypeLabel.setFont(getFont().derive(Font.BOLD));
        syncTypeField = new ObjectChoiceField(
                "",
                choices,
                Configuration.syncType
                );
                
        LabelField rangeLabel = new LabelField(
                StaticDataHelper.getLanguage("conf_range")
                );
        rangeLabel.setFont(getFont().derive(Font.BOLD));
            
        rangePastField = new NumericChoiceField(
                StaticDataHelper.getLanguage("conf_range-past") + "  ",
            1,// begin
            3,// end
            1,// increment
            Configuration.rangePast - 1);
        rangePastField.setEditable(false);
        
        rangeFutureField = new NumericChoiceField(
                StaticDataHelper.getLanguage("conf_range-future") + "  ",
            1,// begin
            6,// end
            1,// increment
            Configuration.rangeFuture - 1);
        rangeFutureField.setEditable(false);
        
        
        //TODO language?
        choices = new String [] {
                "Disabled",
                "Info",
                "Debug",
                "Trace"
        };

        LabelField logLevelLabel = new LabelField(
                        StaticDataHelper.getLanguage("conf_log-level")
                        );
        logLevelLabel.setFont(getFont().derive(Font.BOLD));
        logLevelField = new ObjectChoiceField(
                "",
                choices,
                Configuration.logLevel
                );
        
        // /////////////////////////////////////////////////////////////////////////
        
        final SeparatorField separator1 = new SeparatorField();
        final SeparatorField separator2 = new SeparatorField();
        final SeparatorField separator3 = new SeparatorField();
        
        // /////////////////////////////////////////////////////////////////////////
        
        VerticalFieldManager vfm;
        HorizontalFieldManager hfm;
        
        if (Customization.getInstance().syncUriEditable()) {
            add(serverUriLabel);
            add(serverUriField);
        }
        add(new LabelField(""));
        add(usernameLabel);
        add(usernameField);
        add(new LabelField(""));
        add(passwordLabel);
        add(passwordField);
        
        add(new LabelField(""));
        add(separator1);
        add(new LabelField(""));
        
        
        vfm = new VerticalFieldManager();
        hfm = new HorizontalFieldManager();
        if (Customization.getInstance().contactEnabled()) {
            hfm.add(enableSyncContactField);
            vfm.add(enableSyncContactLabel);
            vfm.add(contactSourceUriField);
        }
        if (SyncClient.checkCustom(SyncClient.CUSTOMS_COMPANY_DIRECTORY))
        {
            vfm.add(cdfield);
        }
        hfm.add(vfm);
        add(hfm);
        

        add(new LabelField(""));
       
        if (Customization.getInstance().calendarEnabled()) {
            vfm = new VerticalFieldManager();
            hfm = new HorizontalFieldManager();
            hfm.add(enableSyncCalendarField);
            vfm.add(enableSyncCalendarLabel);
            vfm.add(calendarSourceUriField);
            hfm.add(vfm);
            add(hfm);
            add(new LabelField(""));
        }
        
        if (Customization.getInstance().taskEnabled()) {
            vfm = new VerticalFieldManager();
            hfm = new HorizontalFieldManager();
            hfm.add(enableSyncTaskField);
            vfm.add(enableSyncTaskLabel);
            vfm.add(taskSourceUriField);
            hfm.add(vfm);
            add(hfm);
            add(new LabelField(""));
        }

        if (Customization.getInstance().noteEnabled()) {
            vfm = new VerticalFieldManager();
            hfm = new HorizontalFieldManager();
            hfm.add(enableSyncNoteField);
            vfm.add(enableSyncNoteLabel);
            vfm.add(noteSourceUriField);
            hfm.add(vfm);
            add(hfm);
            
            add(new LabelField(""));
        }

        add(separator2);
        add(new LabelField(""));
        
        vfm = new VerticalFieldManager();
        hfm = new HorizontalFieldManager();
        hfm.add(enablePollingPIMField);
        vfm.add(enablePollingPIMLabel);
        vfm.add(pollingPIMIntervalField);
        hfm.add(vfm);
        add(hfm);
        add(new LabelField(""));
        
        hfm = new HorizontalFieldManager();
        hfm.add(syncTypeLabel);
        hfm.add(syncTypeField);
        add(hfm);
        
        add(new LabelField(""));
        
        add(rangeLabel);
        add(rangePastField);
        add(rangeFutureField);
        
        add(new LabelField(""));
        add(separator3);
        add(new LabelField(""));
        
        hfm = new HorizontalFieldManager();
        hfm.add(logLevelLabel);
        hfm.add(logLevelField);
        add(hfm);

    }
    
    /**
     * Adds "Save" menu item to the menu on Configuration screen
     * 
     * @param Menu :
     *            menu to which save menuitem is to be added
     * @param int :
     *            instance
     * @return void.
     */
    public void makeMenu(Menu menu, int instance)
    {
        menu.add(saveMenu);
        menu.add(backMenu);
    }

    /**
     * Invoked on user hitting the escape button. It prompts the user with a
     * dialog, if he wants to save/discard/cancel the changes in configuration
     * 
     * @return true iff the configuration screen got closed
     */
    public boolean onClose()
    {
        if (isDataChanged())
        {
            final int response = Dialog.ask(Dialog.D_SAVE, StaticDataHelper
                            .getLanguage("dialog_save"));
            
            if (Dialog.SAVE == response)
            {
                saveConfiguration();
                app.hideScreen(ConfigurationScreen.this);
                // Reset the log level to the new level
                Log.setLogLevel(logLevelField.getSelectedIndex());
            }
            else if (Dialog.DISCARD == response)
            {
                app.popScreen(ConfigurationScreen.this);
            }
            else if (Dialog.CANCEL == response)
            {
                return false;
            }
            return true;
        } else {
            app.popScreen(ConfigurationScreen.this);
            return true;
        }
        
    }

    
    /**
     * Invoked when a user selects the "Save" option from the menu in the
     * configuration screen or from the dialog when the user exits the
     * configuration screen. It saves changes on the configuration into the
     * persistant store
     * 
     */
    public boolean saveConfiguration()
    {
        Log.debug("ConfigurationScreen.saveConfiguration(): Entered");
        
        String serverUri                        = null;
        String username                         = null;
        String password                         = null;
        boolean enableSyncContact       = false;
        boolean enableSyncCalendar      = false;
        boolean enableSyncTask          = false;
        boolean enableSyncNote          = false;
        String contactSourceUri         = null;
        String calendarSourceUri        = null;
        String taskSourceUri            = null;
        String noteSourceUri            = null;
        boolean enablePollingPIM        = false;
        int pollingPIMInterval          = 30;
        int rangePast                   = 1;
        int rangeFuture                 = 6;
        int syncType                    = 0;
        int logLevel                    = 0;
        
        if (Customization.getInstance().syncUriEditable()) {
            serverUri = serverUriField.getText().trim();
        } else {
            serverUri = Configuration.syncUrl;
        }

        username  = usernameField.getText().trim();
        password  = passwordField.getText().trim();
        
        contactSourceUri  = contactSourceUriField.getText().trim();
        calendarSourceUri = calendarSourceUriField.getText().trim();
        taskSourceUri     = taskSourceUriField.getText().trim();
        noteSourceUri     = noteSourceUriField.getText().trim();
        
        enableSyncContact = enableSyncContactField.getChecked();
        enableSyncCalendar = enableSyncCalendarField.getChecked();
        enableSyncTask = enableSyncTaskField.getChecked();
        enableSyncNote = enableSyncNoteField.getChecked();
        
        enablePollingPIM = enablePollingPIMField.getChecked();
        pollingPIMInterval = pollingPIMIntervalField.getSelectedValue();
        
        rangePast = rangePastField.getSelectedValue();
        rangeFuture = rangeFutureField.getSelectedValue();
        
        syncType = syncTypeField.getSelectedIndex();
        
        logLevel = logLevelField.getSelectedIndex();
        
        if (serverUri == null || username == null || password == null || contactSourceUri == null
            || calendarSourceUri == null || taskSourceUri == null || noteSourceUri == null)
        {
                SyncClient.showMessage(
                        StaticDataHelper.getLanguage("message_config-error") + ": " +
                        StaticDataHelper.getLanguage("message_config-error-null")
                    );
            return false;
        }
        else if (
            serverUri.equals("") ||
            username.equals("") ||
            password.equals("")
            )
        {
            SyncClient.showMessage(
                    StaticDataHelper.getLanguage("message_config-error") + ": " +
                    StaticDataHelper.getLanguage("message_config-error-required")
                );
            return false;
        }
        else if (
                (enableSyncContact      && contactSourceUri.equals("")) ||
                (enableSyncCalendar     && calendarSourceUri.equals("")) ||
                (enableSyncTask         && taskSourceUri.equals("")) ||
                (enableSyncNote         && noteSourceUri.equals(""))
                )
            {
                SyncClient.showMessage(
                        StaticDataHelper.getLanguage("message_config-error") + ": " +
                        StaticDataHelper.getLanguage("message_config-error-incomplete")
                    );
                return false;
            }
        else if ((serverUri.indexOf("http") != 0) && (serverUri.indexOf("https") != 0))
        {
            SyncClient.showMessage(
                    StaticDataHelper.getLanguage("message_config-error") + ": " +
                    StaticDataHelper.getLanguage("message_config-error-server")
                );
            return false;
        }

        // Okay, save the configuration
        Configuration.syncUrl = serverUri;
        Configuration.username = username;
        Configuration.password = password;
        
        Configuration.syncContact = enableSyncContact;
        Configuration.contactSourceUri = contactSourceUri;
        
        Configuration.syncCalendar = enableSyncCalendar;
        Configuration.calendarSourceUri = calendarSourceUri;
        
        Configuration.syncTask = enableSyncTask;
        Configuration.taskSourceUri = taskSourceUri;
        
        Configuration.syncNote = enableSyncNote;
        Configuration.noteSourceUri = noteSourceUri;
        
        Configuration.syncType = syncType;
        Configuration.logLevel = logLevel;

        Configuration.pollingPIMInterval = pollingPIMInterval;
        Configuration.enablePollingPIM = enablePollingPIM;
        
        Configuration.rangePast = rangePast;
        Configuration.rangeFuture = rangeFuture;

        if (Configuration.save(app) != Configuration.CONF_OK)
        {
            SyncClient.showMessage(
                        StaticDataHelper.getLanguage("message_config-error") + ": " +
                        StaticDataHelper.getLanguage("message_config-error-save")
                        );
                return false;
        }

        // ------------------------------------------------------------------------------------------------------------------------------------
        
        if (enablePollingPIM)
        {
            app.enablePollingPIM();
        }
        else
        {
            app.disablePollingPIM();
        }

        //app.restartCtp();
        
        return true;
    }

    /**
     * This method checks if a single value in the configuration has changed.
     * The old and the new values are compared. The comparison is done on the
     * trimmed values.
     *
     * @param oldValue the old value of the setting
     * @param newValue the new value of the setting
     * @return true iff oldValue is different than newValue
     */
    private boolean isValueChanged(String oldValue, String newValue) {
        if (oldValue == null && newValue == null) {
            return false;
        } else if (oldValue == null || newValue == null) {
            return true;
        } else {
            String oldCleaned = oldValue.trim();
            String newCleaned = newValue.trim();
            return !newCleaned.equals(oldCleaned);
        }
    }

    /**
     * This method checks if at least one parameter in the configuration has
     * changed. The purpose of the method is to check if the user should be
     * asked if the new configuration needs to be saved, discarded or the
     * operation cancelled (@see onClose).
     *
     * @return true iff at least one configuration parameter has changed
     */
    private boolean isDataChanged() {
 
        boolean isUriChanged = false;
        if (Customization.getInstance().syncUriEditable()) {
            isUriChanged =isValueChanged(Configuration.syncUrl,
                                         serverUriField.getText());
        }

        boolean isUidChanged = isValueChanged(Configuration.username,
                                              usernameField.getText());
        boolean isPwdChanged = isValueChanged(Configuration.password,
                                              passwordField.getText());

        boolean isSyncContactChanged = false;
        if (Customization.getInstance().contactEnabled()) {
            isSyncContactChanged = Configuration.syncContact !=
                                   enableSyncContactField.getChecked();
        }

        boolean isSyncCalendarChanged = false;
        if (Customization.getInstance().calendarEnabled()) {
            isSyncCalendarChanged = Configuration.syncCalendar !=
                                    enableSyncCalendarField.getChecked();
        }

        boolean isSyncTaskChanged = false;
        if (Customization.getInstance().taskEnabled()) {
            isSyncTaskChanged = Configuration.syncTask !=
                                enableSyncTaskField.getChecked();
        }

        boolean isSyncNoteChanged = false;
        if (Customization.getInstance().noteEnabled()) {
            isSyncNoteChanged = Configuration.syncNote !=
                                enableSyncNoteField.getChecked();
        }

        boolean isEnablePollingPIMChanged = Configuration.enablePollingPIM !=
                                            enablePollingPIMField.getChecked();
        
        int currentPollMin = pollingPIMIntervalField.getSelectedValue();
        boolean isPollingPIMIntervalChanged = Configuration.pollingPIMInterval != currentPollMin;
                                              
        boolean isSyncTypeChanged = Configuration.syncType != syncTypeField.getSelectedIndex();

        boolean isLogLevelChanged = Configuration.logLevel != logLevelField.getSelectedIndex();
        
        Log.debug("isUriChanged: " + isUriChanged);
        Log.debug("isUidChanged: " + isUidChanged);
        Log.debug("isPwdChanged: " + isPwdChanged);
        Log.debug("isSyncContactChanged: " + isSyncContactChanged);
        Log.debug("isSyncCalendarChanged: " + isSyncCalendarChanged);
        Log.debug("isSyncTaskChanged: " + isSyncTaskChanged);
        Log.debug("isSyncNoteChanged: " + isSyncNoteChanged);
        Log.debug("isEnablePollingPIMChanged: " + isEnablePollingPIMChanged);
        Log.debug("Initial PPIM interval: " + Configuration.pollingPIMInterval);
        Log.debug("New PPIM interval: " + currentPollMin);
        Log.debug("isPollingPIMIntervalChanged: " + isPollingPIMIntervalChanged);
        Log.debug("isSyncTypeChanged: " + isSyncTypeChanged);
        Log.debug("isLogLevelChanged: " + isLogLevelChanged);
        
        return isUriChanged ||
               isUidChanged||
               isPwdChanged||
               isSyncContactChanged||
               isSyncCalendarChanged||
               isSyncTaskChanged||
               isSyncNoteChanged||
               isEnablePollingPIMChanged||
               isPollingPIMIntervalChanged||
               isSyncTypeChanged ||
               isLogLevelChanged;
    }


}
